apiAttachAvailable = u'API beschikbaar'
apiAttachNotAvailable = u'Niet beschikbaar'
apiAttachPendingAuthorization = u'Wacht op autorisatie'
apiAttachRefused = u'Geweigerd'
apiAttachSuccess = u'Succes'
apiAttachUnknown = u'Onbekend'
budDeletedFriend = u'Van de vriendenlijst verwijderd'
budFriend = u'Vriend'
budNeverBeenFriend = u'Nooit op de vriendenlijst gestaan'
budPendingAuthorization = u'Wacht op autorisatie'
budUnknown = u'Onbekend'
cfrBlockedByRecipient = u'Gesprek geblokkeerd door ontvanger'
cfrMiscError = u'Overige fout'
cfrNoCommonCodec = u'No common codec found'
cfrNoProxyFound = u'Geen proxy gevonden'
cfrNotAuthorizedByRecipient = u'Huidige gebruiker niet door ontvanger geautoriseerd'
cfrRecipientNotFriend = u'Ontvanger geen vriend'
cfrRemoteDeviceError = u'Problem with remote sound device'
cfrSessionTerminated = u'Session terminated'
cfrSoundIOError = u'Geluidsfout'
cfrSoundRecordingError = u'Geluidsopnamefout'
cfrUnknown = u'Onbekend'
cfrUserDoesNotExist = u'Gebruiker/telefoonnummer bestaat niet'
cfrUserIsOffline = u'User is offline'
chsAllCalls = u'Erfenisdialoog'
chsDialog = u'Dialoog'
chsIncomingCalls = u'Multi moet geaccepteerd worden'
chsLegacyDialog = u'Erfenisdialoog'
chsMissedCalls = u'Dialoog'
chsMultiNeedAccept = u'Multi moet geaccepteerd worden'
chsMultiSubscribed = u'Multi ingeschreven'
chsOutgoingCalls = u'Multi ingeschreven'
chsUnknown = u'Onbekend'
chsUnsubscribed = u'Niet ingeschreven'
clsBusy = u'Busy'
clsCancelled = u'Cancelled'
clsEarlyMedia = u'Early Media aan het afspelen'
clsFailed = u'Sorry, call failed!'
clsFinished = u'Be\xebindigd'
clsInProgress = u'Call in Progress'
clsLocalHold = u'Zelf in de wacht gezet'
clsMissed = u'Missed'
clsOnHold = u'On Hold'
clsRefused = u'Geweigerd'
clsRemoteHold = u'In de wacht gezet door ontvanger'
clsRinging = u'Calling'
clsRouting = u'Bezig met routeren'
clsTransferred = u'Onbekend'
clsTransferring = u'Onbekend'
clsUnknown = u'Onbekend'
clsUnplaced = u'Nooit plaatsgevonden'
clsVoicemailBufferingGreeting = u'Begroeting wordt gebufferd'
clsVoicemailCancelled = u'Voicemail is geannuleerd'
clsVoicemailFailed = u'Voicemail Failed'
clsVoicemailPlayingGreeting = u'Begroeting wordt afgespeeld'
clsVoicemailRecording = u'Recording Voicemail'
clsVoicemailSent = u'Voicemail is verzonden'
clsVoicemailUploading = u'Voicemail aan het uploaden'
cltIncomingP2P = u'Inkomend peer-to-peer-gesprek'
cltIncomingPSTN = u'Inkomend telefoongesprek'
cltOutgoingP2P = u'Uitgaand peer-to-peer-gesprek'
cltOutgoingPSTN = u'Uitgaand telefoongesprek'
cltUnknown = u'Onbekend'
cmeAddedMembers = u'Toegevoegde leden'
cmeCreatedChatWith = u'Chat aangemaakt met'
cmeEmoted = u'Onbekend'
cmeLeft = u'Over'
cmeSaid = u'Gezegd'
cmeSawMembers = u'Leden gezien'
cmeSetTopic = u'Topic instellen'
cmeUnknown = u'Onbekend'
cmsRead = u'Gelezen'
cmsReceived = u'Ontvangen'
cmsSending = u'Sending...'
cmsSent = u'Verzonden'
cmsUnknown = u'Onbekend'
conConnecting = u'Connecting'
conOffline = u'Offline'
conOnline = u'Online'
conPausing = u'Pauze'
conUnknown = u'Onbekend'
cusAway = u'Away'
cusDoNotDisturb = u'Do Not Disturb'
cusInvisible = u'Invisible'
cusLoggedOut = u'Logged Out'
cusNotAvailable = u'Niet beschikbaar'
cusOffline = u'Offline'
cusOnline = u'Online'
cusSkypeMe = u'SkypeMe'
cusUnknown = u'Onbekend'
cvsBothEnabled = u'Video verzenden en ontvangen'
cvsNone = u'Geen video'
cvsReceiveEnabled = u'Video ontvangen'
cvsSendEnabled = u'Video verzenden'
cvsUnknown = u''
grpAllFriends = u'Alle vrienden'
grpAllUsers = u'Alle gebruikers'
grpCustomGroup = u'Aangepast'
grpOnlineFriends = u'Online vrienden'
grpPendingAuthorizationFriends = u'Wacht op autorisatie'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'Onlangs gecontacteerde gebruikers'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'Skype-vrienden'
grpSkypeOutFriends = u'SkypeOut-vrienden'
grpUngroupedFriends = u'Ongegroepeerde vrienden'
grpUnknown = u'Onbekend'
grpUsersAuthorizedByMe = u'Door mij geautoriseerd'
grpUsersBlockedByMe = u'Door mij geblokkeerd'
grpUsersWaitingMyAuthorization = u'Wacht op mijn autorisatie'
leaAddDeclined = u'Toevoeging afgeslagen'
leaAddedNotAuthorized = u'Toegevoegde moet geautoriseerd zijn'
leaAdderNotFriend = u'Toevoeger moet vriend zijn'
leaUnknown = u'Onbekend'
leaUnsubscribe = u'Niet ingeschreven'
leaUserIncapable = u'Gebruiker incapabel'
leaUserNotFound = u'Gebruiker niet gevonden'
olsAway = u'Away'
olsDoNotDisturb = u'Do Not Disturb'
olsNotAvailable = u'Niet beschikbaar'
olsOffline = u'Offline'
olsOnline = u'Online'
olsSkypeMe = u'Skype Me'
olsSkypeOut = u'SkypeOut'
olsUnknown = u'Onbekend'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'Female'
usexMale = u'Male'
usexUnknown = u'Onbekend'
vmrConnectError = u'Verbindingsfout'
vmrFileReadError = u'Bestandsleesfout'
vmrFileWriteError = u'Bestandsschrijffout'
vmrMiscError = u'Overige fout'
vmrNoError = u'Geen fout'
vmrNoPrivilege = u'Geen voicemailbevoegdheid'
vmrNoVoicemail = u'Voicemail bestaat niet'
vmrPlaybackError = u'Afspeelfout'
vmrRecordingError = u'Opnamefout'
vmrUnknown = u'Onbekend'
vmsBlank = u'Leeg'
vmsBuffering = u'Bezig met bufferen'
vmsDeleting = u'Bezig met verwijderen'
vmsDownloading = u'Aan het downloaden'
vmsFailed = u'Mislukt'
vmsNotDownloaded = u'Niet gedownload'
vmsPlayed = u'Afgespeeld'
vmsPlaying = u'Bezig met afspelen'
vmsRecorded = u'Opgenomen'
vmsRecording = u'Recording'
vmsUnknown = u'Onbekend'
vmsUnplayed = u'Niet afgespeeld'
vmsUploaded = u'Ge\xfcpload'
vmsUploading = u'Bezig met uploaden'
vmtCustomGreeting = u'Aangepaste begroeting'
vmtDefaultGreeting = u'Standaard begroeting'
vmtIncoming = u'Incoming'
vmtOutgoing = u'Uitgaand'
vmtUnknown = u'Onbekend'
vssAvailable = u'Beschikbaar'
vssNotAvailable = u'Niet beschikbaar'
vssPaused = u'Pauze'
vssRejected = u'Afgewezen'
vssRunning = u'Actief'
vssStarting = u'Bezig met starten'
vssStopping = u'Bezig met stoppen'
vssUnknown = u'Onbekend'



"""
Low-level Skype API definitions.

This subpackage imports one of the:

- `Skype4Py.api.darwin`
- `Skype4Py.api.posix`
- `Skype4Py.api.windows`

modules based on the current platform.

Name of the imported module in available in the `platform` variable.

The modules implement the low-level Skype API and define options
for the `Skype.__init__` constructor.
"""
__docformat__ = 'restructuredtext en'


import sys
import threading
import logging

from Skype4Py.utils import *
from Skype4Py.enums import apiAttachUnknown
from Skype4Py.errors import SkypeAPIError


__all__ = ['Command', 'SkypeAPINotifier', 'SkypeAPI']


DEFAULT_PROTOCOL = 5
DEFAULT_FRIENDLYNAME = u'Skype4Py'
DEFAULT_TIMEOUT = 30000


class Command(object):
    """Represents an API command. Use `Skype.Command` to instantiate.

    To send a command to Skype, use `Skype.SendCommand`.
    """

    def __init__(self, Command, Expected=u'', Blocking=False, Timeout=DEFAULT_TIMEOUT, Id=-1):
        """Use `Skype.Command` to instantiate the object instead of doing it directly.
        """

        self.Blocking = Blocking
        """If set to True, `Skype.SendCommand` will block until the reply is received.
        
        :type: bool"""

        self.Command = tounicode(Command)
        """Command string.
        
        :type: unicode"""

        self.Expected = tounicode(Expected)
        """Expected reply.
        
        :type: unicode"""

        self.Id = Id
        """Command Id.
        
        :type: int"""

        self.Reply = u''
        """Reply after the command has been sent and Skype has replied.
        
        :type: unicode"""

        self.Timeout = Timeout
        """Timeout if Blocking == True.
        
        :type: int"""

    def __repr__(self):
        return '<%s with Command=%s, Blocking=%s, Reply=%s, Id=%s>' % \
            (object.__repr__(self)[1:-1], repr(self.Command), self.Blocking, repr(self.Reply), self.Id)

    def timeout2float(self):
        """A wrapper for `api.timeout2float` function. Returns the converted
        `Timeout` property.
        """
        return timeout2float(self.Timeout)


class SkypeAPINotifier(object):
    def attachment_changed(self, status):
        pass

    def notification_received(self, notification):
        pass
        
    def sending_command(self, command):
        pass

    def reply_received(self, command):
        pass


class SkypeAPIBase(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self, name='Skype4Py API thread')
        self.setDaemon(True)
        if not hasattr(self, 'logger'):
            # Create a logger if the subclass hasn't done it already.
            self.logger = logging.getLogger('Skype4Py.api.SkypeAPIBase')
        self.friendly_name = DEFAULT_FRIENDLYNAME
        self.protocol = DEFAULT_PROTOCOL
        self.commands = {}
        # This lock is the main mechanism to make Skype4Py thread-safe.
        self.rlock = threading.RLock()
        self.notifier = SkypeAPINotifier()
        self.attachment_status = apiAttachUnknown
        self.logger.info('opened')

    def _not_implemented(self):
        raise SkypeAPIError('Function not implemented')
        
    def set_notifier(self, notifier):
        self.notifier = notifier
        
    def push_command(self, command):
        self.acquire()
        try:
            if command.Id < 0:
                command.Id = 0
                while command.Id in self.commands:
                    command.Id += 1
            elif command.Id in self.commands:
                raise SkypeAPIError('Command Id conflict')
            self.commands[command.Id] = command
        finally:
            self.release()

    def pop_command(self, id_):
        self.acquire()
        try:
            try:
                return self.commands.pop(id_)
            except KeyError:
                return None
        finally:
            self.release()

    def acquire(self):
        self.rlock.acquire()
        
    def release(self):
        self.rlock.release()

    def close(self):
        self.logger.info('closed')

    def set_friendly_name(self, friendly_name):
        self.friendly_name = friendly_name

    def set_attachment_status(self, attachment_status):
        if attachment_status != self.attachment_status:
            self.logger.info('attachment: %s', attachment_status)
            self.attachment_status = attachment_status
            self.notifier.attachment_changed(attachment_status)

    def attach(self, timeout, wait=True):
        self._not_implemented()

    def is_running(self):
        self._not_implemented()

    def startup(self, minimized, nosplash):
        self._not_implemented()

    def shutdown(self):
        self._not_implemented()

    def send_command(self, command):
        self._not_implemented()

    def security_context_enabled(self, context):
        self._not_implemented()

    def enable_security_context(self, context):
        self._not_implemented()

    def allow_focus(self, timeout):
        pass


def timeout2float(timeout):
    """Converts a timeout expressed in milliseconds or seconds into a timeout expressed
    in seconds using a floating point number.
    
    :Parameters:
      timeout : int, long or float
        The input timeout. Assumed to be expressed in number of
        milliseconds if the type is int or long. For float, assumed
        to be a number of seconds (or fractions thereof).
    
    :return: The timeout expressed in number of seconds (or fractions thereof).
    :rtype: float
    """
    if isinstance(timeout, float):
        return timeout
    return timeout / 1000.0


def finalize_opts(opts):
    """Convinient function called after popping all options from a dictionary.
    If there are any items left, a TypeError exception is raised listing all
    unexpected keys in the error message.
    """
    if opts:
        raise TypeError('Unexpected option(s): %s' % ', '.join(opts.keys()))


# Select appropriate low-level Skype API module
if getattr(sys, 'skype4py_setup', False):
    # dummy for the setup.py run
    SkypeAPI = lambda **Options: None
    platform = ''
elif sys.platform.startswith('win'):
    from windows import SkypeAPI
    platform = 'windows'
elif sys.platform == 'darwin':
    from darwin import SkypeAPI
    platform = 'darwin'
else:
    from posix import SkypeAPI
    platform = 'posix'


# Note. py2exe will include the darwin but not the posix module. This seems to be the case
# solely because of the "posix" name. It might be a bug in py2exe or modulefinder caused
# by a failed attempt to import a "posix" module by the os module. If this is encountered
# during modulefinder scanning, the Skype4Py.api.posix is simply ignored.
#
# That being said ideally we would like to exclude both of them but I couldn't find a way
# to cause py2exe to skip them. I think py2exe should expose mechanisms to cooperate with
# extension modules aware of its existence.



"""Calls, conferences.
"""
__docformat__ = 'restructuredtext en'


from types import NoneType

from utils import *
from enums import *


class DeviceMixin(object):
    def _Device(self, Name, DeviceType=None, Set=NoneType):
        args = args2dict(self._Property(Name, Cache=False))
        if Set is NoneType:
            for dev, value in args.items():
                try:
                    args[dev] = int(value)
                except ValueError:
                    pass
            if DeviceType is None:
                return args
            return args.get(DeviceType, None)
        elif DeviceType is None:
            raise TypeError('DeviceType must be specified if Set is used')
        if Set:
            args[DeviceType] = tounicode(Set)
        else:
            args.pop(DeviceType, None)
        for dev, value in args.items():
            args[dev] = quote(value, True)
        self._Alter('SET_%s' % Name,
                    ', '.join('%s=%s' % item for item in args.items()))

    def CaptureMicDevice(self, DeviceType=None, Set=NoneType):
        """Queries or sets the mic capture device.

        :Parameters:
          DeviceType : `enums`.callIoDeviceType* or None
            Mic capture device type.
          Set
            Value the device should be set to or None if it should be deactivated.

        Querying all active devices:
            Devices = CaptureMicDevice()
          
          Returns a mapping of device types to their values. Only active devices are
          returned.
          
        Querying a specific device:
            Value = CaptureMicDevice(DeviceType)
          
          Returns a device value for the given DeviceType.
          
        Setting a device value:
            CaptureMicDevice(DeviceType, Value)
          
          If Value is None, the device will be deactivated.

        :note: This command functions for active calls only.
        """
        return self._Device('CAPTURE_MIC', DeviceType, Set)

    def InputDevice(self, DeviceType=None, Set=NoneType):
        """Queries or sets the sound input device.

        :Parameters:
          DeviceType : `enums`.callIoDeviceType* or None
            Sound input device type.
          Set
            Value the device should be set to or None if it should be deactivated.

        Querying all active devices:
            Devices = InputDevice()
          
          Returns a mapping of device types to their values. Only active devices are
          returned.
          
        Querying a specific device:
            Value = InputDevice(DeviceType)
          
          Returns a device value for the given DeviceType.
          
        Setting a device value:
            InputDevice(DeviceType, Value)

          If Value is None, the device will be deactivated.

        :note: This command functions for active calls only.
        """
        return self._Device('INPUT', DeviceType, Set)

    def OutputDevice(self, DeviceType=None, Set=NoneType):
        """Queries or sets the sound output device.

        :Parameters:
          DeviceType : `enums`.callIoDeviceType* or None
            Sound output device type.
          Set
            Value the device should be set to or None if it should be deactivated.

        Querying all active devices:
            Devices = OutputDevice()
          
          Returns a mapping of device types to their values. Only active devices are
          returned.
          
        Querying a specific device:
            Value = OutputDevice(DeviceType)
          
          Returns a device value for the given DeviceType.
          
        Setting a device value:
            OutputDevice(DeviceType, Value)

          If Value is None, the device will be deactivated.

        :note: This command functions for active calls only.
        """
        return self._Device('OUTPUT', DeviceType, Set)


class Call(Cached, DeviceMixin):
    """Represents a voice/video call.
    """
    _ValidateHandle = int

    def __repr__(self):
        return Cached.__repr__(self, 'Id')

    def _Alter(self, AlterName, Args=None):
        return self._Owner._Alter('CALL', self.Id, AlterName, Args)

    def _Init(self):
        self._MakeOwner()

    def _Property(self, PropName, Set=None, Cache=True):
        return self._Owner._Property('CALL', self.Id, PropName, Set, Cache)

    def Answer(self):
        """Answers the call.
        """
        #self._Property('STATUS', 'INPROGRESS')
        self._Alter('ANSWER')

    def CanTransfer(self, Target):
        """Queries if a call can be transferred to a contact or phone number.

        :Parameters:
          Target : str
            Skypename or phone number the call is to be transferred to.

        :return: True if call can be transferred, False otherwise.
        :rtype: bool
        """
        return self._Property('CAN_TRANSFER %s' % Target) == 'TRUE'

    def Finish(self):
        """Ends the call.
        """
        #self._Property('STATUS', 'FINISHED')
        self._Alter('END', 'HANGUP')

    def Forward(self):
        """Forwards a call.
        """
        self._Alter('END', 'FORWARD_CALL')

    def Hold(self):
        """Puts the call on hold.
        """
        #self._Property('STATUS', 'ONHOLD')
        self._Alter('HOLD')

    def Join(self, Id):
        """Joins with another call to form a conference.

        :Parameters:
          Id : int
            Call Id of the other call to join to the conference.

        :return: Conference object.
        :rtype: `Conference`
        """
        #self._Alter('JOIN_CONFERENCE', Id)
        reply = self._Owner._DoCommand('SET CALL %s JOIN_CONFERENCE %s' % (self.Id, Id),
            'CALL %s CONF_ID' % self.Id)
        return Conference(self._Owner, reply.split()[-1])

    def MarkAsSeen(self):
        """Marks the call as seen.
        """
        self.Seen = True

    def RedirectToVoicemail(self):
        """Redirects a call to voicemail.
        """
        self._Alter('END', 'REDIRECT_TO_VOICEMAIL')

    def Resume(self):
        """Resumes the held call.
        """
        #self.Answer()
        self._Alter('RESUME')

    def StartVideoReceive(self):
        """Starts video receive.
        """
        self._Alter('START_VIDEO_RECEIVE')

    def StartVideoSend(self):
        """Starts video send.
        """
        self._Alter('START_VIDEO_SEND')

    def StopVideoReceive(self):
        """Stops video receive.
        """
        self._Alter('STOP_VIDEO_RECEIVE')

    def StopVideoSend(self):
        """Stops video send.
        """
        self._Alter('STOP_VIDEO_SEND')

    def Transfer(self, *Targets):
        """Transfers a call to one or more contacts or phone numbers.

        :Parameters:
          Targets : str
            one or more phone numbers or Skypenames the call is being transferred to.

        :note: You can transfer an incoming call to a group by specifying more than one target,
               first one of the group to answer will get the call.
        :see: `CanTransfer`
        """
        self._Alter('TRANSFER', ', '.join(Targets))

    def _GetConferenceId(self):
        return int(self._Property('CONF_ID'))

    ConferenceId = property(_GetConferenceId,
    doc="""Conference Id.

    :type: int
    """)

    def _GetDatetime(self):
        from datetime import datetime
        return datetime.fromtimestamp(self.Timestamp)

    Datetime = property(_GetDatetime,
    doc="""Date and time of the call.

    :type: datetime.datetime

    :see: `Timestamp`
    """)

    def _SetDTMF(self, Value):
        self._Alter('DTMF', Value)

    DTMF = property(fset=_SetDTMF,
    doc="""Set this property to send DTMF codes. Permitted symbols are: [0..9, #, \*]. 

    :type: str

    :note: This command functions for active calls only.
    """)

    def _GetDuration(self):
        return int(self._Property('DURATION', Cache=False))

    Duration = property(_GetDuration,
    doc="""Duration of the call in seconds.

    :type: int
    """)

    def _GetFailureReason(self):
        return int(self._Property('FAILUREREASON'))

    FailureReason = property(_GetFailureReason,
    doc="""Call failure reason. Read if `Status` == `enums.clsFailed`.

    :type: `enums`.cfr*
    """)

    def _GetForwardedBy(self):
        return str(self._Property('FORWARDED_BY'))

    ForwardedBy = property(_GetForwardedBy,
    doc="""Skypename of the user who forwarded a call.

    :type: str
    """)

    def _GetId(self):
        return self._Handle

    Id = property(_GetId,
    doc="""Call Id.

    :type: int
    """)

    def _GetInputStatus(self):
        return (self._Property('VAA_INPUT_STATUS') == 'TRUE')

    InputStatus = property(_GetInputStatus,
    doc="""True if call voice input is enabled.

    :type: bool
    """)

    def _GetParticipants(self):
        count = int(self._Property('CONF_PARTICIPANTS_COUNT'))
        return ParticipantCollection(self, xrange(count))

    Participants = property(_GetParticipants,
    doc="""Participants of a conference call not hosted by the user.

    :type: `ParticipantCollection`
    """)

    def _GetPartnerDisplayName(self):
        return self._Property('PARTNER_DISPNAME')

    PartnerDisplayName = property(_GetPartnerDisplayName,
    doc="""The DisplayName of the remote caller.

    :type: unicode
    """)

    def _GetPartnerHandle(self):
        return str(self._Property('PARTNER_HANDLE'))

    PartnerHandle = property(_GetPartnerHandle,
    doc="""The Skypename of the remote caller.

    :type: str
    """)

    def _GetPstnNumber(self):
        return str(self._Property('PSTN_NUMBER'))

    PstnNumber = property(_GetPstnNumber,
    doc="""PSTN number of the call.

    :type: str
    """)

    def _GetPstnStatus(self):
        return self._Property('PSTN_STATUS')

    PstnStatus = property(_GetPstnStatus,
    doc="""PSTN number status.

    :type: unicode
    """)

    def _GetRate(self):
        return int(self._Property('RATE'))

    Rate = property(_GetRate,
    doc="""Call rate. Expressed using `RatePrecision`. If you're just interested in the call rate
    expressed in current currency, use `RateValue` instead.

    :type: int

    :see: `RateCurrency`, `RatePrecision`, `RateToText`, `RateValue`
    """)

    def _GetRateCurrency(self):
        return self._Property('RATE_CURRENCY')

    RateCurrency = property(_GetRateCurrency,
    doc="""Call rate currency.

    :type: unicode

    :see: `Rate`, `RatePrecision`, `RateToText`, `RateValue`
    """)

    def _GetRatePrecision(self):
        return int(self._Property('RATE_PRECISION'))

    RatePrecision = property(_GetRatePrecision,
    doc="""Call rate precision. Expressed as a number of times the call rate has to be divided by 10.

    :type: int

    :see: `Rate`, `RateCurrency`, `RateToText`, `RateValue`
    """)

    def _GetRateToText(self):
        return (u'%s %.3f' % (self.RateCurrency, self.RateValue)).strip()

    RateToText = property(_GetRateToText,
    doc="""Returns the call rate as a text with currency and properly formatted value.

    :type: unicode

    :see: `Rate`, `RateCurrency`, `RatePrecision`, `RateValue`
    """)

    def _GetRateValue(self):
        if self.Rate < 0:
            return 0.0
        return float(self.Rate) / (10 ** self.RatePrecision)

    RateValue = property(_GetRateValue,
    doc="""Call rate value. Expressed in current currency.

    :type: float

    :see: `Rate`, `RateCurrency`, `RatePrecision`, `RateToText`
    """)

    def _GetSeen(self):
        return (self._Property('SEEN') == 'TRUE')

    def _SetSeen(self, Value):
        self._Property('SEEN', cndexp(Value, 'TRUE', 'FALSE'))

    Seen = property(_GetSeen, _SetSeen,
    doc="""Queries/sets the seen status of the call. True if the call was seen, False otherwise.

    :type: bool

    :note: You cannot alter the call seen status from seen to unseen.
    """)

    def _GetStatus(self):
        return str(self._Property('STATUS'))

    def _SetStatus(self, Value):
        self._Property('STATUS', str(Value))

    Status = property(_GetStatus, _SetStatus,
    doc="""The call status.

    :type: `enums`.cls*
    """)

    def _GetSubject(self):
        return self._Property('SUBJECT')

    Subject = property(_GetSubject,
    doc="""Call subject.

    :type: unicode
    """)

    def _GetTargetIdentity(self):
        return str(self._Property('TARGET_IDENTITY'))

    TargetIdentity = property(_GetTargetIdentity,
    doc="""Target number for incoming SkypeIn calls.

    :type: str
    """)

    def _GetTimestamp(self):
        return float(self._Property('TIMESTAMP'))

    Timestamp = property(_GetTimestamp,
    doc="""Call date and time expressed as a timestamp.

    :type: float

    :see: `Datetime`
    """)

    def _GetTransferActive(self):
        return self._Property('TRANSFER_ACTIVE') == 'TRUE'

    TransferActive = property(_GetTransferActive,
    doc="""Returns True if the call has been transferred.

    :type: bool
    """)

    def _GetTransferredBy(self):
        return str(self._Property('TRANSFERRED_BY'))

    TransferredBy = property(_GetTransferredBy,
    doc="""Returns the Skypename of the user who transferred the call.

    :type: str
    """)

    def _GetTransferredTo(self):
        return str(self._Property('TRANSFERRED_TO'))

    TransferredTo = property(_GetTransferredTo,
    doc="""Returns the Skypename of the user or phone number the call has been transferred to.

    :type: str
    """)

    def _GetTransferStatus(self):
        return str(self._Property('TRANSFER_STATUS'))

    TransferStatus = property(_GetTransferStatus,
    doc="""Returns the call transfer status.

    :type: `enums`.cls*
    """)

    def _GetType(self):
        return str(self._Property('TYPE'))

    Type = property(_GetType,
    doc="""Call type.

    :type: `enums`.clt*
    """)

    def _GetVideoReceiveStatus(self):
        return str(self._Property('VIDEO_RECEIVE_STATUS'))

    VideoReceiveStatus = property(_GetVideoReceiveStatus,
    doc="""Call video receive status.

    :type: `enums`.vss*
    """)

    def _GetVideoSendStatus(self):
        return str(self._Property('VIDEO_SEND_STATUS'))

    VideoSendStatus = property(_GetVideoSendStatus,
    doc="""Call video send status.

    :type: `enums`.vss*
    """)

    def _GetVideoStatus(self):
        return str(self._Property('VIDEO_STATUS'))

    VideoStatus = property(_GetVideoStatus,
    doc="""Call video status.

    :type: `enums`.cvs*
    """)

    def _GetVmAllowedDuration(self):
        return int(self._Property('VM_ALLOWED_DURATION'))

    VmAllowedDuration = property(_GetVmAllowedDuration,
    doc="""Returns the permitted duration of a voicemail in seconds.

    :type: int
    """)

    def _GetVmDuration(self):
        return int(self._Property('VM_DURATION'))

    VmDuration = property(_GetVmDuration,
    doc="""Returns the duration of a voicemail.

    :type: int
    """)


class CallCollection(CachedCollection):
    _CachedType = Call


class Participant(Cached):
    """Represents a conference call participant.
    """
    _ValidateHandle = int

    def __repr__(self):
        return Cached.__repr__(self, 'Id', 'Idx', 'Handle')

    def _Property(self, Prop):
        # Prop: 0 = user name, 1 = call type, 2 = call status, 3 = display name
        reply = self._Owner._Property('CONF_PARTICIPANT %d' % self.Idx)
        return chop(reply, 3)[Prop]

    def _GetCall(self):
        return self._Owner

    Call = property(_GetCall,
    doc="""Call object.

    :type: `Call`
    """)

    def _GetCallStatus(self):
        return str(self._Property(2))

    CallStatus = property(_GetCallStatus,
    doc="""Call status of a participant in a conference call.

    :type: `enums`.cls*
    """)

    def _GetCallType(self):
        return str(self._Property(1))

    CallType = property(_GetCallType,
    doc="""Call type in a conference call.

    :type: `enums`.clt*
    """)

    def _GetDisplayName(self):
        return self._Property(3)

    DisplayName = property(_GetDisplayName,
    doc="""DisplayName of a participant in a conference call.

    :type: unicode
    """)

    def _GetHandle(self):
        return str(self._Property(0))

    Handle = property(_GetHandle,
    doc="""Skypename of a participant in a conference call.

    :type: str
    """)

    def _GetId(self):
        return self._Owner.Id

    Id = property(_GetId,
    doc="""Call Id.

    :type: int
    """)

    def _GetIdx(self):
        return self._Handle

    Idx = property(_GetIdx,
    doc="""Call participant index.

    :type: int
    """)


class ParticipantCollection(CachedCollection):
    _CachedType = Participant


class Conference(Cached):
    """Represents a conference call.
    """
    _ValidateHandle = int

    def __repr__(self):
        return Cached.__repr__(self, 'Id')

    def Finish(self):
        """Finishes a conference so all active calls have the status
        `enums.clsFinished`.
        """
        for c in self._GetCalls():
            c.Finish()

    def Hold(self):
        """Places all calls in a conference on hold so all active calls
        have the status `enums.clsLocalHold`.
        """
        for c in self._GetCalls():
            c.Hold()

    def Resume(self):
        """Resumes a conference that was placed on hold so all active calls
        have the status `enums.clsInProgress`.
        """
        for c in self._GetCalls():
            c.Resume()

    def _GetActiveCalls(self):
        return CallCollection(self._Owner, (x.Id for x in self._Owner.ActiveCalls if x.ConferenceId == self.Id))

    ActiveCalls = property(_GetActiveCalls,
    doc="""Active calls with the same conference ID.

    :type: `CallCollection`
    """)

    def _GetCalls(self):
        return CallCollection(self._Owner, (x.Id for x in self._Owner.Calls() if x.ConferenceId == self.Id))

    Calls = property(_GetCalls,
    doc="""Calls with the same conference ID.

    :type: `CallCollection`
    """)

    def _GetId(self):
        return self._Handle

    Id = property(_GetId,
    doc="""Id of a conference.

    :type: int
    """)


class ConferenceCollection(CachedCollection):
    _CachedType = Conference



apiAttachAvailable = u'API tilgjengelig'
apiAttachNotAvailable = u'Ikke tilgjengelig'
apiAttachPendingAuthorization = u'Venter p\xe5 \xe5 bli godkjent'
apiAttachRefused = u'Avsl\xe5tt'
apiAttachSuccess = u'Vellykket'
apiAttachUnknown = u'Ukjent'
budDeletedFriend = u'Slettet fra kontaktlisten'
budFriend = u'Venn'
budNeverBeenFriend = u'Aldri v\xe6rt i kontaktlisten'
budPendingAuthorization = u'Venter p\xe5 \xe5 bli godkjent'
budUnknown = u'Ukjent'
cfrBlockedByRecipient = u'Anrop blokkert av mottaker'
cfrMiscError = u'Diverse feil'
cfrNoCommonCodec = u'Ingen felles kodek'
cfrNoProxyFound = u'Finner ingen proxy'
cfrNotAuthorizedByRecipient = u'Gjeldende bruker er ikke godkjent av mottakeren.'
cfrRecipientNotFriend = u'Mottakeren er ikke en venn'
cfrRemoteDeviceError = u'Problem med ekstern lydenhet'
cfrSessionTerminated = u'\xd8kt avsluttet'
cfrSoundIOError = u'I/U-feil for lyd'
cfrSoundRecordingError = u'Lydinnspillingsfeil'
cfrUnknown = u'Ukjent'
cfrUserDoesNotExist = u'Bruker/telefonnummer finnes ikke'
cfrUserIsOffline = u'Hun eller han er frakoblet'
chsAllCalls = u'Foreldet dialogboks'
chsDialog = u'Dialogboks'
chsIncomingCalls = u'Flere m\xe5 godkjenne'
chsLegacyDialog = u'Foreldet dialogboks'
chsMissedCalls = u'Dialogboks'
chsMultiNeedAccept = u'Flere m\xe5 godkjenne'
chsMultiSubscribed = u'Flere abonnert'
chsOutgoingCalls = u'Flere abonnert'
chsUnknown = u'Ukjent'
chsUnsubscribed = u'Ikke abonnert'
clsBusy = u'Opptatt'
clsCancelled = u'Avbryt'
clsEarlyMedia = u'Spiller tidlige media (Early Media)'
clsFailed = u'beklager, anropet feilet!'
clsFinished = u'Avsluttet'
clsInProgress = u'Anrop p\xe5g\xe5r'
clsLocalHold = u'Lokalt parkert samtale'
clsMissed = u'Tapt anrop'
clsOnHold = u'Parkert'
clsRefused = u'Avsl\xe5tt'
clsRemoteHold = u'Eksternt parkert samtale'
clsRinging = u'Anrop'
clsRouting = u'Ruting'
clsTransferred = u'Ukjent'
clsTransferring = u'Ukjent'
clsUnknown = u'Ukjent'
clsUnplaced = u'Aldri plassert'
clsVoicemailBufferingGreeting = u'Bufrer talepostintro'
clsVoicemailCancelled = u'Talepostmelding er annullert'
clsVoicemailFailed = u'Talepost feilet'
clsVoicemailPlayingGreeting = u'Spiller av hilsen'
clsVoicemailRecording = u'Tar opp talepost'
clsVoicemailSent = u'Talepostmelding er sendt'
clsVoicemailUploading = u'Laster opp talepost'
cltIncomingP2P = u'Innkommende P2P-anrop'
cltIncomingPSTN = u'Innkommende telefonanrop'
cltOutgoingP2P = u'Utg\xe5ende P2P-anrop'
cltOutgoingPSTN = u'Utg\xe5ende telefonanrop'
cltUnknown = u'Ukjent'
cmeAddedMembers = u'Medlemmer som er lagt til'
cmeCreatedChatWith = u'Opprettet tekstsamtale med'
cmeEmoted = u'Ukjent'
cmeLeft = u'Forlatt'
cmeSaid = u'Sa'
cmeSawMembers = u'Medlemmer som ble sett'
cmeSetTopic = u'Angitt emne'
cmeUnknown = u'Ukjent'
cmsRead = u'Lest'
cmsReceived = u'Mottatt'
cmsSending = u'Sender...'
cmsSent = u'Sendt'
cmsUnknown = u'Ukjent'
conConnecting = u'Kobler til'
conOffline = u'Frakoblet'
conOnline = u'P\xe5logget'
conPausing = u'Settes i pause'
conUnknown = u'Ukjent'
cusAway = u'Borte'
cusDoNotDisturb = u'Opptatt'
cusInvisible = u'Vis som Usynlig'
cusLoggedOut = u'Frakoblet'
cusNotAvailable = u'Ikke tilgjengelig'
cusOffline = u'Frakoblet'
cusOnline = u'P\xe5logget'
cusSkypeMe = u'Skype Meg'
cusUnknown = u'Ukjent'
cvsBothEnabled = u'Videosending og -mottak'
cvsNone = u'Ingen video'
cvsReceiveEnabled = u'Videomottak'
cvsSendEnabled = u'Videosending'
cvsUnknown = u''
grpAllFriends = u'Alle venner'
grpAllUsers = u'Alle brukere'
grpCustomGroup = u'Tilpasset'
grpOnlineFriends = u'Elektroniske venner'
grpPendingAuthorizationFriends = u'Venter p\xe5 \xe5 bli godkjent'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'Nylig kontaktede brukere'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'Skype-venner'
grpSkypeOutFriends = u'SkypeOut-venner'
grpUngroupedFriends = u'Usorterte venner'
grpUnknown = u'Ukjent'
grpUsersAuthorizedByMe = u'Godkjent av meg'
grpUsersBlockedByMe = u'Blokkert av meg'
grpUsersWaitingMyAuthorization = u'Venter p\xe5 min godkjenning'
leaAddDeclined = u'Tillegging avvist'
leaAddedNotAuthorized = u'Den som legger til, m\xe5 v\xe6re autorisert'
leaAdderNotFriend = u'Den som legger til, m\xe5 v\xe6re en venn'
leaUnknown = u'Ukjent'
leaUnsubscribe = u'Ikke abonnert'
leaUserIncapable = u'Bruker forhindret'
leaUserNotFound = u'Finner ikke bruker'
olsAway = u'Borte'
olsDoNotDisturb = u'Opptatt'
olsNotAvailable = u'Ikke tilgjengelig'
olsOffline = u'Frakoblet'
olsOnline = u'P\xe5logget'
olsSkypeMe = u'Skype Meg'
olsSkypeOut = u'SkypeOut'
olsUnknown = u'Ukjent'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'Kvinne'
usexMale = u'Mann'
usexUnknown = u'Ukjent'
vmrConnectError = u'Koblingsfeil'
vmrFileReadError = u'Fillesingsfeil'
vmrFileWriteError = u'Filskrivingsfeil'
vmrMiscError = u'Diverse feil'
vmrNoError = u'Ingen feil'
vmrNoPrivilege = u'Intet talepostprivilegium'
vmrNoVoicemail = u'Ingen slik talepost'
vmrPlaybackError = u'Avspillingsfeil'
vmrRecordingError = u'Innspillingsfeil'
vmrUnknown = u'Ukjent'
vmsBlank = u'Tom'
vmsBuffering = u'Bufring'
vmsDeleting = u'Sletter'
vmsDownloading = u'Laster ned'
vmsFailed = u'Mislyktes'
vmsNotDownloaded = u'Ikke nedlastet'
vmsPlayed = u'Spilt av'
vmsPlaying = u'Spiller av'
vmsRecorded = u'Innspilt'
vmsRecording = u'Tar opp talepost'
vmsUnknown = u'Ukjent'
vmsUnplayed = u'Ikke avspilt'
vmsUploaded = u'Lastet opp'
vmsUploading = u'Laster opp'
vmtCustomGreeting = u'Tilpasset hilsen'
vmtDefaultGreeting = u'Standardhilsen'
vmtIncoming = u'Innkommende talepost'
vmtOutgoing = u'Utg\xe5ende'
vmtUnknown = u'Ukjent'
vssAvailable = u'Tilgjengelig'
vssNotAvailable = u'Ikke tilgjengelig'
vssPaused = u'Satt i pause'
vssRejected = u'Avvist'
vssRunning = u'Kj\xf8rer'
vssStarting = u'Starter'
vssStopping = u'Stanser'
vssUnknown = u'Ukjent'



"""
Skype4Py constants.

**Warning!** Remember that all constants defined here are available at
the `Skype4Py` package level and should be accessed from there.

Example:
   
.. python::

    import Skype4Py

    status = Skype4Py.apiAttachSuccess
"""
__docformat__ = 'restructuredtext en'


#{ Attachment status
apiAttachUnknown = -1
apiAttachSuccess = 0
apiAttachPendingAuthorization = 1
apiAttachRefused = 2
apiAttachNotAvailable = 3
apiAttachAvailable = 0x8001


#{ Connection status
conUnknown = 'UNKNOWN'
conOffline = 'OFFLINE'
conConnecting = 'CONNECTING'
conPausing = 'PAUSING'
conOnline = 'ONLINE'


#{ User status
cusUnknown = 'UNKNOWN'
cusOffline = 'OFFLINE'
cusOnline = 'ONLINE'
cusAway = 'AWAY'
cusNotAvailable = 'NA'
cusDoNotDisturb = 'DND'
cusInvisible = 'INVISIBLE'
cusLoggedOut = 'LOGGEDOUT'
cusSkypeMe = 'SKYPEME'


#{ Call failure reason
cfrUnknown = -1
cfrMiscError = 1
cfrUserDoesNotExist = 2
cfrUserIsOffline = 3
cfrNoProxyFound = 4
cfrSessionTerminated = 5
cfrNoCommonCodec = 6
cfrSoundIOError = 7
cfrRemoteDeviceError = 8
cfrBlockedByRecipient = 9
cfrRecipientNotFriend = 10
cfrNotAuthorizedByRecipient = 11
cfrSoundRecordingError = 12


#{ Call status
clsUnknown = 'NOT_AVAILABLE'
clsUnplaced = 'UNPLACED'
clsRouting = 'ROUTING'
clsEarlyMedia = 'EARLYMEDIA'
clsFailed = 'FAILED'
clsRinging = 'RINGING'
clsInProgress = 'INPROGRESS'
clsOnHold = 'ONHOLD'
clsFinished = 'FINISHED'
clsMissed = 'MISSED'
clsRefused = 'REFUSED'
clsBusy = 'BUSY'
clsCancelled = 'CANCELLED'
clsLocalHold = 'REDIAL_PENDING'
clsRemoteHold = 'REMOTEHOLD'
clsVoicemailBufferingGreeting = 'VM_BUFFERING_GREETING'
clsVoicemailPlayingGreeting = 'VM_PLAYING_GREETING'
clsVoicemailRecording = 'VM_RECORDING'
clsVoicemailUploading = 'VM_UPLOADING'
clsVoicemailSent = 'VM_SENT'
clsVoicemailCancelled = 'VM_CANCELLED'
clsVoicemailFailed = 'VM_FAILED'
clsTransferring = 'TRANSFERRING'
clsTransferred = 'TRANSFERRED'


#{ Call type
cltUnknown = 'UNKNOWN'
cltIncomingPSTN = 'INCOMING_PSTN'
cltOutgoingPSTN = 'OUTGOING_PSTN'
cltIncomingP2P = 'INCOMING_P2P'
cltOutgoingP2P = 'OUTGOING_P2P'


#{ Call history
chsAllCalls = 'ALL'
chsMissedCalls = 'MISSED'
chsIncomingCalls = 'INCOMING'
chsOutgoingCalls = 'OUTGOING'


#{ Call video status
cvsUnknown = 'UNKNOWN'
cvsNone = 'VIDEO_NONE'
cvsSendEnabled = 'VIDEO_SEND_ENABLED'
cvsReceiveEnabled = 'VIDEO_RECV_ENABLED'
cvsBothEnabled = 'VIDEO_BOTH_ENABLED'


#{ Call video send status
vssUnknown = 'UNKNOWN'
vssNotAvailable = 'NOT_AVAILABLE'
vssAvailable = 'AVAILABLE'
vssStarting = 'STARTING'
vssRejected = 'REJECTED'
vssRunning = 'RUNNING'
vssStopping = 'STOPPING'
vssPaused = 'PAUSED'


#{ Call IO device type
callIoDeviceTypeUnknown = 'UNKNOWN'
callIoDeviceTypeSoundcard = 'SOUNDCARD'
callIoDeviceTypePort = 'PORT'
callIoDeviceTypeFile = 'FILE'


#{ Chat message type
cmeUnknown = 'UNKNOWN'
cmeCreatedChatWith = 'CREATEDCHATWITH'
cmeSawMembers = 'SAWMEMBERS'
cmeAddedMembers = 'ADDEDMEMBERS'
cmeSetTopic = 'SETTOPIC'
cmeSaid = 'SAID'
cmeLeft = 'LEFT'
cmeEmoted = 'EMOTED'
cmePostedContacts = 'POSTEDCONTACTS'
cmeGapInChat = 'GAP_IN_CHAT'
cmeSetRole = 'SETROLE'
cmeKicked = 'KICKED'
cmeKickBanned = 'KICKBANNED'
cmeSetOptions = 'SETOPTIONS'
cmeSetPicture = 'SETPICTURE'
cmeSetGuidelines = 'SETGUIDELINES'
cmeJoinedAsApplicant = 'JOINEDASAPPLICANT'


#{ Chat message status
cmsUnknown = 'UNKNOWN'
cmsSending = 'SENDING'
cmsSent = 'SENT'
cmsReceived = 'RECEIVED'
cmsRead = 'READ'


#{ User sex
usexUnknown = 'UNKNOWN'
usexMale = 'MALE'
usexFemale = 'FEMALE'


#{ Buddy status
budUnknown = -1
budNeverBeenFriend = 0
budDeletedFriend = 1
budPendingAuthorization = 2
budFriend = 3


#{ Online status
olsUnknown = 'UNKNOWN'
olsOffline = 'OFFLINE'
olsOnline = 'ONLINE'
olsAway = 'AWAY'
olsNotAvailable = 'NA'
olsDoNotDisturb = 'DND'
olsInvisible = 'INVISIBLE'
olsSkypeOut = 'SKYPEOUT'
olsSkypeMe = 'SKYPEME'


#{ Chat leave reason
leaUnknown = ''
leaUserNotFound = 'USER_NOT_FOUND'
leaUserIncapable = 'USER_INCAPABLE'
leaAdderNotFriend = 'ADDER_MUST_BE_FRIEND'
leaAddedNotAuthorized = 'ADDED_MUST_BE_AUTHORIZED'
leaAddDeclined = 'ADD_DECLINED'
leaUnsubscribe = 'UNSUBSCRIBE'


#{ Chat status
chsUnknown = 'UNKNOWN'
chsLegacyDialog = 'LEGACY_DIALOG'
chsDialog = 'DIALOG'
chsMultiNeedAccept = 'MULTI_NEED_ACCEPT'
chsMultiSubscribed = 'MULTI_SUBSCRIBED'
chsUnsubscribed = 'UNSUBSCRIBED'


#{ Voicemail type
vmtUnknown = 'UNKNOWN'
vmtIncoming = 'INCOMING'
vmtDefaultGreeting = 'DEFAULT_GREETING'
vmtCustomGreeting = 'CUSTOM_GREETING'
vmtOutgoing = 'OUTGOING'


#{ Voicemail status
vmsUnknown = 'UNKNOWN'
vmsNotDownloaded = 'NOTDOWNLOADED'
vmsDownloading = 'DOWNLOADING'
vmsUnplayed = 'UNPLAYED'
vmsBuffering = 'BUFFERING'
vmsPlaying = 'PLAYING'
vmsPlayed = 'PLAYED'
vmsBlank = 'BLANK'
vmsRecording = 'RECORDING'
vmsRecorded = 'RECORDED'
vmsUploading = 'UPLOADING'
vmsUploaded = 'UPLOADED'
vmsDeleting = 'DELETING'
vmsFailed = 'FAILED'


#{ Voicemail failure reason
vmrUnknown = 'UNKNOWN'
vmrNoError = 'NOERROR'
vmrMiscError = 'MISC_ERROR'
vmrConnectError = 'CONNECT_ERROR'
vmrNoPrivilege = 'NO_VOICEMAIL_PRIVILEGE'
vmrNoVoicemail = 'NO_SUCH_VOICEMAIL'
vmrFileReadError = 'FILE_READ_ERROR'
vmrFileWriteError = 'FILE_WRITE_ERROR'
vmrRecordingError = 'RECORDING_ERROR'
vmrPlaybackError = 'PLAYBACK_ERROR'


#{ Group type
grpUnknown = 'UNKNOWN'
grpCustomGroup = 'CUSTOM_GROUP'
grpAllUsers = 'ALL_USERS'
grpAllFriends = 'ALL_FRIENDS'
grpSkypeFriends = 'SKYPE_FRIENDS'
grpSkypeOutFriends = 'SKYPEOUT_FRIENDS'
grpOnlineFriends = 'ONLINE_FRIENDS'
grpPendingAuthorizationFriends = 'UNKNOWN_OR_PENDINGAUTH_FRIENDS'
grpRecentlyContactedUsers = 'RECENTLY_CONTACTED_USERS'
grpUsersWaitingMyAuthorization = 'USERS_WAITING_MY_AUTHORIZATION'
grpUsersAuthorizedByMe = 'USERS_AUTHORIZED_BY_ME'
grpUsersBlockedByMe = 'USERS_BLOCKED_BY_ME'
grpUngroupedFriends = 'UNGROUPED_FRIENDS'
grpSharedGroup = 'SHARED_GROUP'
grpProposedSharedGroup = 'PROPOSED_SHARED_GROUP'


#{ Call channel type
cctUnknown = 'UNKNOWN'
cctDatagram = 'DATAGRAM'
cctReliable = 'RELIABLE'


#{ API security context
apiContextUnknown = 0
apiContextVoice = 1
apiContextMessaging = 2
apiContextAccount = 4
apiContextContacts = 8


#{ SMS message type
smsMessageTypeUnknown = 'UNKNOWN'
smsMessageTypeIncoming = 'INCOMING'
smsMessageTypeOutgoing = 'OUTGOING'
smsMessageTypeCCRequest = 'CONFIRMATION_CODE_REQUEST'
smsMessageTypeCCSubmit = 'CONFRIMATION_CODE_SUBMIT'


#{ SMS message status
smsMessageStatusUnknown = 'UNKNOWN'
smsMessageStatusReceived = 'RECEIVED'
smsMessageStatusRead = 'READ'
smsMessageStatusComposing = 'COMPOSING'
smsMessageStatusSendingToServer = 'SENDING_TO_SERVER'
smsMessageStatusSentToServer = 'SENT_TO_SERVER'
smsMessageStatusDelivered = 'DELIVERED'
smsMessageStatusSomeTargetsFailed = 'SOME_TARGETS_FAILED'
smsMessageStatusFailed = 'FAILED'


#{ SMS failure reason
smsFailureReasonUnknown = 'UNKNOWN'
smsFailureReasonMiscError = 'MISC_ERROR'
smsFailureReasonServerConnectFailed = 'SERVER_CONNECT_FAILED'
smsFailureReasonNoSmsCapability = 'NO_SMS_CAPABILITY'
smsFailureReasonInsufficientFunds = 'INSUFFICIENT_FUNDS'
smsFailureReasonInvalidConfirmationCode = 'INVALID_CONFIRMATION_CODE'
smsFailureReasonUserBlocked = 'USER_BLOCKED'
smsFailureReasonIPBlocked = 'IP_BLOCKED'
smsFailureReasonNodeBlocked = 'NODE_BLOCKED'
smsFailureReasonNoSenderIdCapability = 'NO_SENDERID_CAPABILITY'


#{ SMS target status
smsTargetStatusUnknown = 'UNKNOWN'
smsTargetStatusUndefined = 'TARGET_UNDEFINED'
smsTargetStatusAnalyzing = 'TARGET_ANALYZING'
smsTargetStatusAcceptable = 'TARGET_ACCEPTABLE'
smsTargetStatusNotRoutable = 'TARGET_NOT_ROUTABLE'
smsTargetStatusDeliveryPending = 'TARGET_DELIVERY_PENDING'
smsTargetStatusDeliverySuccessful = 'TARGET_DELIVERY_SUCCESSFUL'
smsTargetStatusDeliveryFailed = 'TARGET_DELIVERY_FAILED'


#{ Plug-in context
pluginContextUnknown = 'unknown'
pluginContextChat = 'chat'
pluginContextCall = 'call'
pluginContextContact = 'contact'
pluginContextMyself = 'myself'
pluginContextTools = 'tools'


#{ Plug-in contact type
pluginContactTypeUnknown = 'unknown'
pluginContactTypeAll = 'all'
pluginContactTypeSkype = 'skype'
pluginContactTypeSkypeOut = 'skypeout'


#{ File transfer type
fileTransferTypeIncoming = 'INCOMING'
fileTransferTypeOutgoing = 'OUTGOING'


#{ File transfer status
fileTransferStatusNew = 'NEW'
fileTransferStatusConnecting = 'CONNECTING'
fileTransferStatusWaitingForAccept = 'WAITING_FOR_ACCEPT'
fileTransferStatusTransferring = 'TRANSFERRING'
fileTransferStatusTransferringOverRelay = 'TRANSFERRING_OVER_RELAY'
fileTransferStatusPaused = 'PAUSED'
fileTransferStatusRemotelyPaused = 'REMOTELY_PAUSED'
fileTransferStatusCancelled = 'CANCELLED'
fileTransferStatusCompleted = 'COMPLETED'
fileTransferStatusFailed = 'FAILED'


#{ File transfer failure reason
fileTransferFailureReasonSenderNotAuthorized = 'SENDER_NOT_AUTHORIZED'
fileTransferFailureReasonRemotelyCancelled = 'REMOTELY_CANCELLED'
fileTransferFailureReasonFailedRead = 'FAILED_READ'
fileTransferFailureReasonFailedRemoteRead = 'FAILED_REMOTE_READ'
fileTransferFailureReasonFailedWrite = 'FAILED_WRITE'
fileTransferFailureReasonFailedRemoteWrite = 'FAILED_REMOTE_WRITE'
fileTransferFailureReasonRemoteDoesNotSupportFT = 'REMOTE_DOES_NOT_SUPPORT_FT'
fileTransferFailureReasonRemoteOfflineTooLong = 'REMOTE_OFFLINE_TOO_LONG'


#{ Chat member role
chatMemberRoleUnknown = 'UNKNOWN'
chatMemberRoleCreator = 'CREATOR'
chatMemberRoleMaster = 'MASTER'
chatMemberRoleHelper = 'HELPER'
chatMemberRoleUser = 'USER'
chatMemberRoleListener = 'LISTENER'
chatMemberRoleApplicant = 'APPLICANT'


#{ My chat status
chatStatusUnknown = 'UNKNOWN'
chatStatusConnecting = 'CONNECTING'
chatStatusWaitingRemoteAccept = 'WAITING_REMOTE_ACCEPT'
chatStatusAcceptRequired = 'ACCEPT_REQUIRED'
chatStatusPasswordRequired = 'PASSWORD_REQUIRED'
chatStatusSubscribed = 'SUBSCRIBED'
chatStatusUnsubscribed = 'UNSUBSCRIBED'
chatStatusDisbanded = 'CHAT_DISBANDED'
chatStatusQueuedBecauseChatIsFull = 'QUEUED_BECAUSE_CHAT_IS_FULL'
chatStatusApplicationDenied = 'APPLICATION_DENIED'
chatStatusKicked = 'KICKED'
chatStatusBanned = 'BANNED'
chatStatusRetryConnecting = 'RETRY_CONNECTING'


#{ Chat options
chatOptionJoiningEnabled = 1
chatOptionJoinersBecomeApplicants = 2
chatOptionJoinersBecomeListeners = 4
chatOptionHistoryDisclosed = 8
chatOptionUsersAreListeners = 16
chatOptionTopicAndPictureLockedForUsers = 32


#{ Chat type
chatTypeUnknown = 'UNKNOWN'
chatTypeDialog = 'DIALOG'
chatTypeLegacyDialog = 'LEGACY_DIALOG'
chatTypeLegacyUnsubscribed = 'LEGACY_UNSUBSCRIBED'
chatTypeMultiChat = 'MULTICHAT'
chatTypeSharedGroup = 'SHAREDGROUP'

#{ Window state
wndUnknown = 'UNKNOWN'
wndNormal = 'NORMAL'
wndMinimized = 'MINIMIZED'
wndMaximized = 'MAXIMIZED'
wndHidden = 'HIDDEN'



apiAttachAvailable = u'API dispon\xedvel'
apiAttachNotAvailable = u'N\xe3o dispon\xedvel'
apiAttachPendingAuthorization = u'Autoriza\xe7\xe3o pendente'
apiAttachRefused = u'Recusado'
apiAttachSuccess = u'Sucesso'
apiAttachUnknown = u'Desconhecido'
budDeletedFriend = u'Apagado da Lista de Amigos'
budFriend = u'Amigo'
budNeverBeenFriend = u'Nunca esteve na Lista de Amigos'
budPendingAuthorization = u'Autoriza\xe7\xe3o pendente'
budUnknown = u'Desconhecido'
cfrBlockedByRecipient = u'Chamada bloqueada pelo destinat\xe1rio'
cfrMiscError = u'Erro diverso'
cfrNoCommonCodec = u'Nenhum codec comum'
cfrNoProxyFound = u'Proxy n\xe3o encontrado'
cfrNotAuthorizedByRecipient = u'O utilizador actual n\xe3o est\xe1 autorizado pelo destinat\xe1rio'
cfrRecipientNotFriend = u'Destinat\xe1rio n\xe3o \xe9 um amigo'
cfrRemoteDeviceError = u'Problema com dispositivo de som remoto'
cfrSessionTerminated = u'Sess\xe3o encerrada'
cfrSoundIOError = u'Erro de I/O de som'
cfrSoundRecordingError = u'Erro de grava\xe7\xe3o de voz'
cfrUnknown = u'Desconhecido'
cfrUserDoesNotExist = u'O utilizador/n\xfamero de telefone n\xe3o existe'
cfrUserIsOffline = u'Ela/ele est\xe1 offline'
chsAllCalls = u'Di\xe1logo antigo'
chsDialog = u'Di\xe1logo'
chsIncomingCalls = u'Multi Precisa Aceitar'
chsLegacyDialog = u'Di\xe1logo antigo'
chsMissedCalls = u'Di\xe1logo'
chsMultiNeedAccept = u'Multi Precisa Aceitar'
chsMultiSubscribed = u'Multi Registados'
chsOutgoingCalls = u'Multi Registados'
chsUnknown = u'Desconhecido'
chsUnsubscribed = u'Assinatura eliminada'
clsBusy = u'Ocupado'
clsCancelled = u'Cancelado'
clsEarlyMedia = u'Tocar os dados iniciais (Early Media)'
clsFailed = u'Desculpe, falha na chamada!'
clsFinished = u'Terminado'
clsInProgress = u'Chamada em curso'
clsLocalHold = u'Em espera local'
clsMissed = u'Chamada Perdida'
clsOnHold = u'Em espera'
clsRefused = u'Recusado'
clsRemoteHold = u'Em espera remota'
clsRinging = u'chamadas'
clsRouting = u'A encaminhar'
clsTransferred = u'Desconhecido'
clsTransferring = u'Desconhecido'
clsUnknown = u'Desconhecido'
clsUnplaced = u'Nunca foi adicionado'
clsVoicemailBufferingGreeting = u'A colocar sauda\xe7\xe3o na mem\xf3ria interm\xe9dia'
clsVoicemailCancelled = u'As mensagens (voicemail) foram canceladas'
clsVoicemailFailed = u'Falha na Mensagem de Voz'
clsVoicemailPlayingGreeting = u'A tocar Sauda\xe7\xe3o'
clsVoicemailRecording = u'A gravar Mensagem de Voz'
clsVoicemailSent = u'As mensagens (voicemail) foram enviadas'
clsVoicemailUploading = u'A carregar as mensagens (Voicemail)'
cltIncomingP2P = u'A receber chamada peer-to-peer'
cltIncomingPSTN = u'A receber chamada de voz'
cltOutgoingP2P = u'A efectuar chamada peer-to-peer'
cltOutgoingPSTN = u'A efectuar chamada telef\xf3nica'
cltUnknown = u'Desconhecido'
cmeAddedMembers = u'Membros adicionados'
cmeCreatedChatWith = u'Chat criado com'
cmeEmoted = u'Desconhecido'
cmeLeft = u'Saiu'
cmeSaid = u'Disse'
cmeSawMembers = u'Membros vistos'
cmeSetTopic = u'Definir t\xf3pico'
cmeUnknown = u'Desconhecido'
cmsRead = u'Ler'
cmsReceived = u'Recebido'
cmsSending = u'A enviar...'
cmsSent = u'Enviado'
cmsUnknown = u'Desconhecido'
conConnecting = u'A ligar'
conOffline = u'Offline'
conOnline = u'Online'
conPausing = u'A colocar em pausa'
conUnknown = u'Desconhecido'
cusAway = u'Ausente'
cusDoNotDisturb = u'Ocupado'
cusInvisible = u'Invis\xedvel'
cusLoggedOut = u'Offline'
cusNotAvailable = u'N\xe3o dispon\xedvel'
cusOffline = u'Offline'
cusOnline = u'Online'
cusSkypeMe = u'Skype Me'
cusUnknown = u'Desconhecido'
cvsBothEnabled = u'A enviar e receber v\xeddeo'
cvsNone = u'V\xeddeo n\xe3o dispon\xedvel'
cvsReceiveEnabled = u'A receber v\xeddeo'
cvsSendEnabled = u'A enviar v\xeddeo'
cvsUnknown = u''
grpAllFriends = u'Todos os amigos'
grpAllUsers = u'Todos os utilizadores'
grpCustomGroup = u'Personalizado'
grpOnlineFriends = u'Amigos on-line'
grpPendingAuthorizationFriends = u'Autoriza\xe7\xe3o pendente'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'Utilizadores contactados recentemente'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'Amigos Skype'
grpSkypeOutFriends = u'Amigos SkypeOut'
grpUngroupedFriends = u'Amigos n\xe3o organizados em grupos'
grpUnknown = u'Desconhecido'
grpUsersAuthorizedByMe = u'Autorizado por mim'
grpUsersBlockedByMe = u'Bloqueado por mim'
grpUsersWaitingMyAuthorization = u'\xc0 espera da minha autoriza\xe7\xe3o'
leaAddDeclined = u'Adicionamento recusado'
leaAddedNotAuthorized = u'O adicionado deve ser autorizado'
leaAdderNotFriend = u'Quem adiciona deve ser um amigo'
leaUnknown = u'Desconhecido'
leaUnsubscribe = u'Assinatura eliminada'
leaUserIncapable = u'Utilizador n\xe3o habilitado'
leaUserNotFound = u'Utilizador n\xe3o encontrado'
olsAway = u'Ausente'
olsDoNotDisturb = u'Ocupado'
olsNotAvailable = u'N\xe3o dispon\xedvel'
olsOffline = u'Offline'
olsOnline = u'Online'
olsSkypeMe = u'Skype Me'
olsSkypeOut = u'SkypeOut'
olsUnknown = u'Desconhecido'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'Feminino'
usexMale = u'Masculino'
usexUnknown = u'Desconhecido'
vmrConnectError = u'Erro de liga\xe7\xe3o'
vmrFileReadError = u'Erro na leitura do ficheiro'
vmrFileWriteError = u'Erro a escrever ficheiro'
vmrMiscError = u'Erro diverso'
vmrNoError = u'N\xe3o h\xe1 erro'
vmrNoPrivilege = u'Sem privil\xe9gio de Voicemail'
vmrNoVoicemail = u'N\xe3o existe tal Voicemail'
vmrPlaybackError = u'Erro de reprodu\xe7\xe3o'
vmrRecordingError = u'Erro a gravar'
vmrUnknown = u'Desconhecido'
vmsBlank = u'Em branco'
vmsBuffering = u'A colocar na mem\xf3ria interm\xe9dia'
vmsDeleting = u'A apagar'
vmsDownloading = u'A transferir'
vmsFailed = u'Falhou'
vmsNotDownloaded = u'N\xe3o transferido'
vmsPlayed = u'Reproduzido'
vmsPlaying = u'A reproduzir'
vmsRecorded = u'Gravado'
vmsRecording = u'A gravar Mensagem de Voz'
vmsUnknown = u'Desconhecido'
vmsUnplayed = u'N\xe3o ouvido'
vmsUploaded = u'Carregado'
vmsUploading = u'A carregar'
vmtCustomGreeting = u'Sauda\xe7\xe3o personalizada'
vmtDefaultGreeting = u'Sauda\xe7\xe3o predefinida'
vmtIncoming = u'Receber Voice Mail'
vmtOutgoing = u'De sa\xedda'
vmtUnknown = u'Desconhecido'
vssAvailable = u'Dispon\xedvel'
vssNotAvailable = u'N\xe3o dispon\xedvel'
vssPaused = u'Em pausa'
vssRejected = u'Rejeitada'
vssRunning = u'Em curso'
vssStarting = u'A iniciar'
vssStopping = u'A terminar'
vssUnknown = u'Desconhecido'



apiAttachAvailable = u'API Available'
apiAttachNotAvailable = u'Not Available'
apiAttachPendingAuthorization = u'Pending Authorization'
apiAttachRefused = u'Refused'
apiAttachSuccess = u'Success'
apiAttachUnknown = u'Unknown'
budDeletedFriend = u'Deleted From Friendlist'
budFriend = u'Friend'
budNeverBeenFriend = u'Never Been In Friendlist'
budPendingAuthorization = u'Pending Authorization'
budUnknown = u'Unknown'
cfrBlockedByRecipient = u'Call blocked by recipient'
cfrMiscError = u'Misc error'
cfrNoCommonCodec = u'No common codec found'
cfrNoProxyFound = u'No proxy found'
cfrNotAuthorizedByRecipient = u'Current user not authorized by recipient'
cfrRecipientNotFriend = u'Recipient not a friend'
cfrRemoteDeviceError = u'Problem with remote sound device'
cfrSessionTerminated = u'Session terminated'
cfrSoundIOError = u'Sound I/O error'
cfrSoundRecordingError = u'Sound recording error'
cfrUnknown = u'Unknown'
cfrUserDoesNotExist = u'User/phone number does not exist'
cfrUserIsOffline = u'User is offline'
chsAllCalls = u'Legacy Dialog'
chsDialog = u'Dialog'
chsIncomingCalls = u'Multi Need Accept'
chsLegacyDialog = u'Legacy Dialog'
chsMissedCalls = u'Dialog'
chsMultiNeedAccept = u'Multi Need Accept'
chsMultiSubscribed = u'Multi Subscribed'
chsOutgoingCalls = u'Multi Subscribed'
chsUnknown = u'Unknown'
chsUnsubscribed = u'Unsubscribed'
clsBusy = u'Busy'
clsCancelled = u'Cancelled'
clsEarlyMedia = u'Playing Early Media'
clsFailed = u'Sorry, call failed!'
clsFinished = u'Finished'
clsInProgress = u'Call in Progress'
clsLocalHold = u'On Local Hold'
clsMissed = u'Missed'
clsOnHold = u'On Hold'
clsRefused = u'Refused'
clsRemoteHold = u'On Remote Hold'
clsRinging = u'Calling'
clsRouting = u'Routing'
clsTransferred = u'Unknown'
clsTransferring = u'Unknown'
clsUnknown = u'Unknown'
clsUnplaced = u'Never placed'
clsVoicemailBufferingGreeting = u'Buffering Greeting'
clsVoicemailCancelled = u'Voicemail Has Been Cancelled'
clsVoicemailFailed = u'Voicemail Failed'
clsVoicemailPlayingGreeting = u'Playing Greeting'
clsVoicemailRecording = u'Recording'
clsVoicemailSent = u'Voicemail Has Been Sent'
clsVoicemailUploading = u'Uploading Voicemail'
cltIncomingP2P = u'Incoming Peer-to-Peer Call'
cltIncomingPSTN = u'Incoming Telephone Call'
cltOutgoingP2P = u'Outgoing Peer-to-Peer Call'
cltOutgoingPSTN = u'Outgoing Telephone Call'
cltUnknown = u'Unknown'
cmeAddedMembers = u'Added Members'
cmeCreatedChatWith = u'Created Chat With'
cmeEmoted = u'Unknown'
cmeLeft = u'Left'
cmeSaid = u'Said'
cmeSawMembers = u'Saw Members'
cmeSetTopic = u'Set Topic'
cmeUnknown = u'Unknown'
cmsRead = u'Read'
cmsReceived = u'Received'
cmsSending = u'Sending'
cmsSent = u'Sent'
cmsUnknown = u'Unknown'
conConnecting = u'Connecting'
conOffline = u'Offline'
conOnline = u'Online'
conPausing = u'Pausing'
conUnknown = u'Unknown'
cusAway = u'Away'
cusDoNotDisturb = u'Do Not Disturb'
cusInvisible = u'Invisible'
cusLoggedOut = u'Logged Out'
cusNotAvailable = u'Not Available'
cusOffline = u'Offline'
cusOnline = u'Online'
cusSkypeMe = u'Skype Me'
cusUnknown = u'Unknown'
cvsBothEnabled = u'Video Send and Receive'
cvsNone = u'No Video'
cvsReceiveEnabled = u'Video Receive'
cvsSendEnabled = u'Video Send'
cvsUnknown = u''
grpAllFriends = u'All Friends'
grpAllUsers = u'All Users'
grpCustomGroup = u'Custom'
grpOnlineFriends = u'Online Friends'
grpPendingAuthorizationFriends = u'Pending Authorization'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'Recently Contacted Users'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'Skype Friends'
grpSkypeOutFriends = u'SkypeOut Friends'
grpUngroupedFriends = u'Ungrouped Friends'
grpUnknown = u'Unknown'
grpUsersAuthorizedByMe = u'Authorized By Me'
grpUsersBlockedByMe = u'Blocked By Me'
grpUsersWaitingMyAuthorization = u'Waiting My Authorization'
leaAddDeclined = u'Add Declined'
leaAddedNotAuthorized = u'Added Must Be Authorized'
leaAdderNotFriend = u'Adder Must Be Friend'
leaUnknown = u'Unknown'
leaUnsubscribe = u'Unsubscribed'
leaUserIncapable = u'User Incapable'
leaUserNotFound = u'User Not Found'
olsAway = u'Away'
olsDoNotDisturb = u'Do Not Disturb'
olsNotAvailable = u'Not Available'
olsOffline = u'Offline'
olsOnline = u'Online'
olsSkypeMe = u'SkypeMe'
olsSkypeOut = u'SkypeOut'
olsUnknown = u'Unknown'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'Female'
usexMale = u'Male'
usexUnknown = u'Unknown'
vmrConnectError = u'Connect Error'
vmrFileReadError = u'File Read Error'
vmrFileWriteError = u'File Write Error'
vmrMiscError = u'Misc Error'
vmrNoError = u'No Error'
vmrNoPrivilege = u'No Voicemail Privilege'
vmrNoVoicemail = u'No Such Voicemail'
vmrPlaybackError = u'Playback Error'
vmrRecordingError = u'Recording Error'
vmrUnknown = u'Unknown'
vmsBlank = u'Blank'
vmsBuffering = u'Buffering'
vmsDeleting = u'Deleting'
vmsDownloading = u'Downloading'
vmsFailed = u'Failed'
vmsNotDownloaded = u'Not Downloaded'
vmsPlayed = u'Played'
vmsPlaying = u'Playing'
vmsRecorded = u'Recorded'
vmsRecording = u'Recording Voicemail'
vmsUnknown = u'Unknown'
vmsUnplayed = u'Unplayed'
vmsUploaded = u'Uploaded'
vmsUploading = u'Uploading'
vmtCustomGreeting = u'Custom Greeting'
vmtDefaultGreeting = u'Default Greeting'
vmtIncoming = u'Incoming'
vmtOutgoing = u'Outgoing'
vmtUnknown = u'Unknown'
vssAvailable = u'Available'
vssNotAvailable = u'Not Available'
vssPaused = u'Paused'
vssRejected = u'Rejected'
vssRunning = u'Running'
vssStarting = u'Starting'
vssStopping = u'Stopping'
vssUnknown = u'Unknown'



apiAttachAvailable = u'API \uc0ac\uc6a9\uac00\ub2a5'
apiAttachNotAvailable = u'\uc678\ucd9c \uc911'
apiAttachPendingAuthorization = u'\uc2b9\uc778 \ubcf4\ub958 \uc911'
apiAttachRefused = u'\ud1b5\ud654 \uac70\ubd80'
apiAttachSuccess = u'\uc131\uacf5'
apiAttachUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
budDeletedFriend = u'\uce5c\uad6c\ubaa9\ub85d\uc5d0\uc11c \uc0ad\uc81c\ub428'
budFriend = u'\uce5c\uad6c'
budNeverBeenFriend = u'\uce5c\uad6c\ubaa9\ub85d\uc5d0 \uc785\ub825\ud55c \uc801\uc774 \uc5c6\uc74c'
budPendingAuthorization = u'\uc2b9\uc778 \ubcf4\ub958 \uc911'
budUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
cfrBlockedByRecipient = u'\uc218\uc2e0\uc790\uc5d0 \uc758\ud574 \ud1b5\ud654 \ucc28\ub2e8'
cfrMiscError = u'\uae30\ud0c0 \uc624\ub958'
cfrNoCommonCodec = u'\uc77c\ubc18 \ucf54\ub371 \uc5c6\uc74c'
cfrNoProxyFound = u'\ubc1c\uacac\ub41c \ud504\ub85d\uc2dc \uc5c6\uc74c'
cfrNotAuthorizedByRecipient = u'\ud604 \uc0ac\uc6a9\uc790\ub97c \uc218\uc2e0\uc790\uac00 \uc2b9\uc778\ud558\uc9c0 \uc54a\uc558\uc2b5\ub2c8\ub2e4.'
cfrRecipientNotFriend = u'\uc218\uc2e0\uc790\uac00 \uce5c\uad6c\uac00 \uc544\ub2d8'
cfrRemoteDeviceError = u'\uc6d0\uaca9 \uc0ac\uc6b4\ub4dc IO \uc624\ub958'
cfrSessionTerminated = u'\uc138\uc158 \uc885\ub8cc'
cfrSoundIOError = u'\uc0ac\uc6b4\ub4dc I/O \uc624\ub958'
cfrSoundRecordingError = u'\uc0ac\uc6b4\ub4dc \ub179\uc74c \uc624\ub958'
cfrUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
cfrUserDoesNotExist = u'\uc0ac\uc6a9\uc790/\uc804\ud654\ubc88\ud638\uac00 \uc874\uc7ac\ud558\uc9c0 \uc54a\uc2b5\ub2c8\ub2e4'
cfrUserIsOffline = u'\uc0c1\ub300\ubc29\uc774 \uc624\ud504\ub77c\uc778 \uc0c1\ud0dc\uc785\ub2c8\ub2e4'
chsAllCalls = u'\ub808\uac70\uc2dc \ub2e4\uc774\uc5bc\ub85c\uadf8'
chsDialog = u'\ub2e4\uc774\uc5bc\ub85c\uadf8'
chsIncomingCalls = u'\ub2e4\uc218\uac00 \uc218\ub77d\uc744 \uc694\ud568'
chsLegacyDialog = u'\ub808\uac70\uc2dc \ub2e4\uc774\uc5bc\ub85c\uadf8'
chsMissedCalls = u'\ub2e4\uc774\uc5bc\ub85c\uadf8'
chsMultiNeedAccept = u'\ub2e4\uc218\uac00 \uc218\ub77d\uc744 \uc694\ud568'
chsMultiSubscribed = u'\ub2e4\uc218\uac00 \uac00\uc785\ud568'
chsOutgoingCalls = u'\ub2e4\uc218\uac00 \uac00\uc785\ud568'
chsUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
chsUnsubscribed = u'\ubbf8\uac00\uc785'
clsBusy = u'\ud1b5\ud654 \uc911'
clsCancelled = u'\ucde8\uc18c\ub418\uc5c8\uc74c'
clsEarlyMedia = u'\ucd08\uae30 \ubbf8\ub514\uc5b4(Early Media) \uc7ac\uc0dd\uc911'
clsFailed = u'\uc8c4\uc1a1\ud569\ub2c8\ub2e4. \ud1b5\ud654\uac00 \uc2e4\ud328\ud588\uc2b5\ub2c8\ub2e4!'
clsFinished = u'\uc885\ub8cc'
clsInProgress = u'\ud1b5\ud654 \uc9c4\ud589 \uc911'
clsLocalHold = u'\ud604 \uc0ac\uc6a9\uc790\uac00 \ud1b5\ud654\ubcf4\ub958\ud568'
clsMissed = u'\ud1b5\ud654 \ubabb\ubc1b\uc74c'
clsOnHold = u'\ubcf4\ub958'
clsRefused = u'\ud1b5\ud654 \uac70\ubd80'
clsRemoteHold = u'\uc6d0\uaca9 \ubcf4\ub958\ud568'
clsRinging = u'\uc804\ud654\uac78\uae30'
clsRouting = u'\ub77c\uc6b0\ud305'
clsTransferred = u'\uc54c \uc218 \uc5c6\uc74c'
clsTransferring = u'\uc54c \uc218 \uc5c6\uc74c'
clsUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
clsUnplaced = u'\uc804\ud654\uac00 \uac78\ub9ac\uc9c0 \uc54a\uc74c'
clsVoicemailBufferingGreeting = u'\uc778\uc0ac\ub9d0 \ubc84\ud37c\ub9c1 \uc911'
clsVoicemailCancelled = u'\ubcf4\uc774\uc2a4\uba54\uc77c\uc774 \ucde8\uc18c\ub418\uc5c8\uc2b5\ub2c8\ub2e4'
clsVoicemailFailed = u'\ubcf4\uc774\uc2a4\uba54\uc77c \uc2e4\ud328'
clsVoicemailPlayingGreeting = u'\uc778\uc0ac\ub9d0 \ud50c\ub808\uc774 \uc911'
clsVoicemailRecording = u'\ubcf4\uc774\uc2a4\uba54\uc77c \ub179\uc74c\uc911'
clsVoicemailSent = u'\ubcf4\uc774\uc2a4\uba54\uc77c\uc774 \uc804\uc1a1\ub418\uc5c8\uc2b5\ub2c8\ub2e4'
clsVoicemailUploading = u'\ubcf4\uc774\uc2a4\uba54\uc77c \uc5c5\ub85c\ub4dc \uc911'
cltIncomingP2P = u'\uc218\uc2e0 \ud53c\uc5b4-\ud22c-\ud53c\uc5b4 \ud1b5\ud654'
cltIncomingPSTN = u'\uc218\uc2e0 \uc804\ud654\ud1b5\ud654'
cltOutgoingP2P = u'\ubc1c\uc2e0 \ud53c\uc5b4-\ud22c-\ud53c\uc5b4 \ud1b5\ud654'
cltOutgoingPSTN = u'\ubc1c\uc2e0 \uc804\ud654\ud1b5\ud654'
cltUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
cmeAddedMembers = u'\ud68c\uc6d0\uc744 \ucd94\uac00\ud588\uc74c'
cmeCreatedChatWith = u'\ub2e4\uc74c \uc0c1\ub300\uc640 \ucc44\ud305 \ud615\uc131'
cmeEmoted = u'\uc54c \uc218 \uc5c6\uc74c'
cmeLeft = u'\ub5a0\ub0a8'
cmeSaid = u'\ub9d0\ud55c \ub0b4\uc6a9'
cmeSawMembers = u'\ud68c\uc6d0\uc744 \ubcf4\uc558\uc74c'
cmeSetTopic = u'\uc8fc\uc81c \uc124\uc815'
cmeUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
cmsRead = u'\uc77d\uc74c'
cmsReceived = u'\uc218\uc2e0'
cmsSending = u'\ubcf4\ub0b4\ub294 \uc911...'
cmsSent = u'\uc804\uc1a1\uc644\ub8cc'
cmsUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
conConnecting = u'\uc5f0\uacb0 \uc911'
conOffline = u'\uc624\ud504\ub77c\uc778'
conOnline = u'\uc628\ub77c\uc778'
conPausing = u'\uc77c\uc2dc \uc911\uc9c0'
conUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
cusAway = u'\uc790\ub9ac \ube44\uc6c0'
cusDoNotDisturb = u'\ub2e4\ub978 \uc6a9\ubb34\uc911'
cusInvisible = u'\ud22c\uba85\uc778\uac04'
cusLoggedOut = u'\uc624\ud504\ub77c\uc778'
cusNotAvailable = u'\uc678\ucd9c \uc911'
cusOffline = u'\uc624\ud504\ub77c\uc778'
cusOnline = u'\uc628\ub77c\uc778'
cusSkypeMe = u'Skype Me'
cusUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
cvsBothEnabled = u'\ube44\ub514\uc624 \uc804\uc1a1 \ubc0f \uc218\uc2e0'
cvsNone = u'\ube44\ub514\uc624 \uc5c6\uc74c'
cvsReceiveEnabled = u'\ube44\ub514\uc624 \uc218\uc2e0'
cvsSendEnabled = u'\ube44\ub514\uc624 \uc804\uc1a1'
cvsUnknown = u''
grpAllFriends = u'\ubaa8\ub4e0 \uce5c\uad6c'
grpAllUsers = u'\ubaa8\ub4e0 \uc0ac\uc6a9\uc790'
grpCustomGroup = u'\ub9de\ucda4'
grpOnlineFriends = u'\uc628\ub77c\uc778 \uce5c\uad6c\ub4e4'
grpPendingAuthorizationFriends = u'\uc2b9\uc778 \ubcf4\ub958 \uc911'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'\ucd5c\uadfc\uc5d0 \uc5f0\ub77d\ud55c \uc0ac\uc6a9\uc790'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'Skype \uce5c\uad6c\ub4e4'
grpSkypeOutFriends = u'SkypeOut \uce5c\uad6c\ub4e4'
grpUngroupedFriends = u'\uadf8\ub8f9\uc5d0 \uc18d\ud558\uc9c0 \uc54a\uc740 \uce5c\uad6c\ub4e4'
grpUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
grpUsersAuthorizedByMe = u'\ub0b4\uac00 \uc2b9\uc778\ud55c \uc0ac\uc6a9\uc790'
grpUsersBlockedByMe = u'\ub0b4\uac00 \ucc28\ub2e8\ud55c \uc0ac\uc6a9\uc790'
grpUsersWaitingMyAuthorization = u'\ub0b4 \uc2b9\uc778\uc744 \uae30\ub2e4\ub9ac\ub294 \uc0ac\uc6a9\uc790'
leaAddDeclined = u'\ucd94\uac00\uac00 \uac70\ubd80\ub428'
leaAddedNotAuthorized = u'\ucd94\uac00\ub418\ub294 \ub300\uc0c1\uc774 \ubc18\ub4dc\uc2dc \uc2b9\uc778\ub418\uc5b4\uc57c \ud568'
leaAdderNotFriend = u'\ucd94\uac00\ud558\ub294 \uc0ac\ub78c\uc774 \ubc18\ub4dc\uc2dc \uce5c\uad6c\uc5ec\uc57c \ud568'
leaUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
leaUnsubscribe = u'\ubbf8\uac00\uc785'
leaUserIncapable = u'\uc0ac\uc6a9\uc790 \uc0ac\uc6a9\ubd88\ub2a5'
leaUserNotFound = u'\uc0ac\uc6a9\uc790\ub97c \ucc3e\uc9c0 \ubabb\ud568'
olsAway = u'\uc790\ub9ac \ube44\uc6c0'
olsDoNotDisturb = u'\ub2e4\ub978 \uc6a9\ubb34\uc911'
olsNotAvailable = u'\uc678\ucd9c \uc911'
olsOffline = u'\uc624\ud504\ub77c\uc778'
olsOnline = u'\uc628\ub77c\uc778'
olsSkypeMe = u'Skype Me'
olsSkypeOut = u'SkypeOut'
olsUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'\uc5ec'
usexMale = u'\ub0a8'
usexUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
vmrConnectError = u'\uc5f0\uacb0 \uc624\ub958'
vmrFileReadError = u'\ud30c\uc77c \uc77d\uae30 \uc624\ub958'
vmrFileWriteError = u'\ud30c\uc77c \uc4f0\uae30 \uc624\ub958'
vmrMiscError = u'\uae30\ud0c0 \uc624\ub958'
vmrNoError = u'\uc624\ub958 \uc5c6\uc74c'
vmrNoPrivilege = u'\ubcf4\uc774\uc2a4\uba54\uc77c \uc6b0\uc120\uad8c \uc5c6\uc74c'
vmrNoVoicemail = u'\uadf8\ub7ec\ud55c \ubcf4\uc774\uc2a4\uba54\uc77c\uc774 \uc5c6\uc74c'
vmrPlaybackError = u'\uc7ac\uc0dd \uc624\ub958'
vmrRecordingError = u'\ub179\uc74c \uc624\ub958'
vmrUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
vmsBlank = u'\uba54\uc2dc\uc9c0 \uc5c6\uc74c'
vmsBuffering = u'\ubc84\ud37c\ub9c1'
vmsDeleting = u'\uc0ad\uc81c \uc911'
vmsDownloading = u'\ub2e4\uc6b4\ub85c\ub4dc \uc911'
vmsFailed = u'\uc2e4\ud328\ud568'
vmsNotDownloaded = u'\ub2e4\uc6b4\ub85c\ub4dc \ub418\uc9c0 \uc54a\uc74c'
vmsPlayed = u'\ud50c\ub808\uc774 \ud568'
vmsPlaying = u'\ud50c\ub808\uc774 \uc911'
vmsRecorded = u'\ub179\uc74c\ub428'
vmsRecording = u'\ubcf4\uc774\uc2a4\uba54\uc77c \ub179\uc74c\uc911'
vmsUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
vmsUnplayed = u'\ud50c\ub808\uc774 \uc548\ub428'
vmsUploaded = u'\uc5c5\ub85c\ub4dc\ub428'
vmsUploading = u'\uc5c5\ub85c\ub4dc\uc911'
vmtCustomGreeting = u'\ub9de\ucda4 \uc778\uc0ac\ub9d0'
vmtDefaultGreeting = u'\uc9c0\uc815 \uc778\uc0ac\ub9d0'
vmtIncoming = u'\ubc1b\uc740 \ubcf4\uc774\uc2a4\uba54\uc77c'
vmtOutgoing = u'\ubc1c\uc2e0'
vmtUnknown = u'\uc54c \uc218 \uc5c6\uc74c'
vssAvailable = u'\uc0ac\uc6a9\uac00\ub2a5'
vssNotAvailable = u'\uc678\ucd9c \uc911'
vssPaused = u'\uc77c\uc2dc\uc815\uc9c0'
vssRejected = u'\uac70\ubd80\ub428'
vssRunning = u'\uc791\ub3d9 \uc911'
vssStarting = u'\uc2dc\uc791'
vssStopping = u'\uc815\uc9c0 \uc911'
vssUnknown = u'\uc54c \uc218 \uc5c6\uc74c'



"""Skype client user interface control.
"""
__docformat__ = 'restructuredtext en'


import weakref

from enums import *
from errors import SkypeError
from utils import *


class Client(object):
    """Represents a Skype client. Access using `skype.Skype.Client`.
    """

    def __init__(self, Skype):
        """__init__.

        :Parameters:
          Skype : `Skype`
            Skype
        """
        self._SkypeRef = weakref.ref(Skype)

    def ButtonPressed(self, Key):
        """This command sends a button pressed notification event.

        :Parameters:
          Key : str
            Button key [0-9, A-Z, #, \*, UP, DOWN, YES, NO, SKYPE, PAGEUP, PAGEDOWN].
        """
        self._Skype._DoCommand('BTN_PRESSED %s' % Key)

    def ButtonReleased(self, Key):
        """This command sends a button released notification event.

        :Parameters:
          Key : str
            Button key [0-9, A-Z, #, \*, UP, DOWN, YES, NO, SKYPE, PAGEUP, PAGEDOWN].
        """
        self._Skype._DoCommand('BTN_RELEASED %s' % Key)

    def CreateEvent(self, EventId, Caption, Hint):
        """Creates a custom event displayed in Skype client's events pane.

        :Parameters:
          EventId : unicode
            Unique identifier for the event.
          Caption : unicode
            Caption text.
          Hint : unicode
            Hint text. Shown when mouse hoovers over the event.

        :return: Event object.
        :rtype: `PluginEvent`
        """
        self._Skype._DoCommand('CREATE EVENT %s CAPTION %s HINT %s' % (tounicode(EventId),
            quote(tounicode(Caption)), quote(tounicode(Hint))))
        return PluginEvent(self._Skype, EventId)

    def CreateMenuItem(self, MenuItemId, PluginContext, CaptionText, HintText=u'', IconPath='', Enabled=True,
                       ContactType=pluginContactTypeAll, MultipleContacts=False):
        """Creates custom menu item in Skype client's "Do More" menus.

        :Parameters:
          MenuItemId : unicode
            Unique identifier for the menu item.
          PluginContext : `enums`.pluginContext*
            Menu item context. Allows to choose in which client windows will the menu item appear.
          CaptionText : unicode
            Caption text.
          HintText : unicode
            Hint text (optional). Shown when mouse hoovers over the menu item.
          IconPath : unicode
            Path to the icon (optional).
          Enabled : bool
            Initial state of the menu item. True by default.
          ContactType : `enums`.pluginContactType*
            In case of `enums.pluginContextContact` tells which contacts the menu item should appear
            for. Defaults to `enums.pluginContactTypeAll`.
          MultipleContacts : bool
            Set to True if multiple contacts should be allowed (defaults to False).

        :return: Menu item object.
        :rtype: `PluginMenuItem`
        """
        cmd = 'CREATE MENU_ITEM %s CONTEXT %s CAPTION %s ENABLED %s' % (tounicode(MenuItemId), PluginContext,
            quote(tounicode(CaptionText)), cndexp(Enabled, 'true', 'false'))
        if HintText:
            cmd += ' HINT %s' % quote(tounicode(HintText))
        if IconPath:
            cmd += ' ICON %s' % quote(path2unicode(IconPath))
        if MultipleContacts:
            cmd += ' ENABLE_MULTIPLE_CONTACTS true'
        if PluginContext == pluginContextContact:
            cmd += ' CONTACT_TYPE_FILTER %s' % ContactType
        self._Skype._DoCommand(cmd)
        return PluginMenuItem(self._Skype, MenuItemId, CaptionText, HintText, Enabled)

    def Focus(self):
        """Brings the client window into focus.
        """
        self._Skype._Api.allow_focus(self._Skype.Timeout)
        self._Skype._DoCommand('FOCUS')

    def Minimize(self):
        """Hides Skype application window.
        """
        self._Skype._DoCommand('MINIMIZE')

    def OpenAddContactDialog(self, Username=''):
        """Opens "Add a Contact" dialog.

        :Parameters:
          Username : str
            Optional Skypename of the contact.
        """
        self.OpenDialog('ADDAFRIEND', Username)

    def OpenAuthorizationDialog(self, Username):
        """Opens authorization dialog.

        :Parameters:
          Username : str
            Skypename of the user to authenticate.
        """
        self.OpenDialog('AUTHORIZATION', Username)

    def OpenBlockedUsersDialog(self):
        """Opens blocked users dialog.
        """
        self.OpenDialog('BLOCKEDUSERS')

    def OpenCallHistoryTab(self):
        """Opens call history tab.
        """
        self.OpenDialog('CALLHISTORY')

    def OpenConferenceDialog(self):
        """Opens create conference dialog.
        """
        self.OpenDialog('CONFERENCE')

    def OpenContactsTab(self):
        """Opens contacts tab.
        """
        self.OpenDialog('CONTACTS')

    def OpenDialog(self, Name, *Params):
        """Open dialog. Use this method to open dialogs added in newer Skype versions if there is no
        dedicated method in Skype4Py.

        :Parameters:
          Name : str
            Dialog name.
          Params : unicode
            One or more optional parameters.
        """
        self._Skype._Api.allow_focus(self._Skype.Timeout)
        params = filter(None, (str(Name),) + Params)
        self._Skype._DoCommand('OPEN %s' % tounicode(' '.join(params)))

    def OpenDialpadTab(self):
        """Opens dial pad tab.
        """
        self.OpenDialog('DIALPAD')

    def OpenFileTransferDialog(self, Username, Folder):
        """Opens file transfer dialog.

        :Parameters:
          Username : str
            Skypename of the user.
          Folder : str
            Path to initial directory.
        """
        self.OpenDialog('FILETRANSFER', Username, 'IN', path2unicode(Folder))

    def OpenGettingStartedWizard(self):
        """Opens getting started wizard.
        """
        self.OpenDialog('GETTINGSTARTED')

    def OpenImportContactsWizard(self):
        """Opens import contacts wizard.
        """
        self.OpenDialog('IMPORTCONTACTS')

    def OpenLiveTab(self):
        """OpenLiveTab.
        """
        self.OpenDialog('LIVETAB')

    def OpenMessageDialog(self, Username, Text=u''):
        """Opens "Send an IM Message" dialog.

        :Parameters:
          Username : str
            Message target.
          Text : unicode
            Message text.
        """
        self.OpenDialog('IM', Username, tounicode(Text))

    def OpenOptionsDialog(self, Page=''):
        """Opens options dialog.

        :Parameters:
          Page : str
            Page name to open.

        :see: See https://developer.skype.com/Docs/ApiDoc/OPEN_OPTIONS for known Page values.
        """
        self.OpenDialog('OPTIONS', Page)

    def OpenProfileDialog(self):
        """Opens current user profile dialog.
        """
        self.OpenDialog('PROFILE')

    def OpenSearchDialog(self):
        """Opens search dialog.
        """
        self.OpenDialog('SEARCH')

    def OpenSendContactsDialog(self, Username=''):
        """Opens send contacts dialog.

        :Parameters:
          Username : str
            Optional Skypename of the user.
        """
        self.OpenDialog('SENDCONTACTS', Username)

    def OpenSmsDialog(self, SmsId):
        """Opens SMS window

        :Parameters:
          SmsId : int
            SMS message Id.
        """
        self.OpenDialog('SMS', str(SmsId))

    def OpenUserInfoDialog(self, Username):
        """Opens user information dialog.

        :Parameters:
          Username : str
            Skypename of the user.
        """
        self.OpenDialog('USERINFO', Username)

    def OpenVideoTestDialog(self):
        """Opens video test dialog.
        """
        self.OpenDialog('VIDEOTEST')

    def Shutdown(self):
        """Closes Skype application.
        """
        self._Skype._Api.shutdown()

    def Start(self, Minimized=False, Nosplash=False):
        """Starts Skype application.

        :Parameters:
          Minimized : bool
            If True, Skype is started minimized in system tray.
          Nosplash : bool
            If True, no splash screen is displayed upon startup.
        """
        self._Skype._Api.startup(Minimized, Nosplash)

    def _Get_Skype(self):
        skype = self._SkypeRef()
        if skype:
            return skype
        raise SkypeError('Skype4Py internal error')

    _Skype = property(_Get_Skype)

    def _GetIsRunning(self):
        return self._Skype._Api.is_running()

    IsRunning = property(_GetIsRunning,
    doc="""Tells if Skype client is running.

    :type: bool
    """)

    def _GetWallpaper(self):
        return unicode2path(self._Skype.Variable('WALLPAPER'))

    def _SetWallpaper(self, Value):
        self._Skype.Variable('WALLPAPER', path2unicode(Value))

    Wallpaper = property(_GetWallpaper, _SetWallpaper,
    doc="""Path to client wallpaper bitmap.

    :type: str
    """)

    def _GetWindowState(self):
        return str(self._Skype.Variable('WINDOWSTATE'))

    def _SetWindowState(self, Value):
        self._Skype.Variable('WINDOWSTATE', Value)

    WindowState = property(_GetWindowState, _SetWindowState,
    doc="""Client window state.

    :type: `enums`.wnd*
    """)


class PluginEvent(object):
    """Represents an event displayed in Skype client's events pane.
    """
    def __init__(self, Skype, Id):
        self._Skype = Skype
        self._Id = tounicode(Id)

    def __repr__(self):
        return '<%s with Id=%s>' % (object.__repr__(self)[1:-1], repr(self.Id))

    def Delete(self):
        """Deletes the event from the events pane in the Skype client.
        """
        self._Skype._DoCommand('DELETE EVENT %s' % self.Id)

    def _GetId(self):
        return self._Id

    Id = property(_GetId,
    doc="""Unique event Id.

    :type: unicode
    """)


class PluginMenuItem(object):
    """Represents a menu item displayed in Skype client's "Do More" menus.
    """
    def __init__(self, Skype, Id, Caption, Hint, Enabled):
        self._Skype = Skype
        self._Id = tounicode(Id)
        self._CacheDict = {}
        self._CacheDict['CAPTION'] = tounicode(Caption)
        self._CacheDict['HINT'] = tounicode(Hint)
        self._CacheDict['ENABLED'] = cndexp(Enabled, u'TRUE', u'FALSE')

    def __repr__(self):
        return '<%s with Id=%s>' % (object.__repr__(self)[1:-1], repr(self.Id))

    def _Property(self, PropName, Set=None):
        if Set is None:
            return self._CacheDict[PropName]
        self._Skype._Property('MENU_ITEM', self.Id, PropName, Set)
        self._CacheDict[PropName] = unicode(Set)

    def Delete(self):
        """Removes the menu item from the "Do More" menus.
        """
        self._Skype._DoCommand('DELETE MENU_ITEM %s' % self.Id)

    def _GetCaption(self):
        return self._Property('CAPTION')

    def _SetCaption(self, Value):
        self._Property('CAPTION', tounicode(Value))

    Caption = property(_GetCaption, _SetCaption,
    doc="""Menu item caption text.

    :type: unicode
    """)

    def _GetEnabled(self):
        return (self._Property('ENABLED') == 'TRUE')

    def _SetEnabled(self, Value):
        self._Property('ENABLED', cndexp(Value, 'TRUE', 'FALSE'))

    Enabled = property(_GetEnabled, _SetEnabled,
    doc="""Defines whether the menu item is enabled when a user launches Skype. If no value is defined,
    the menu item will be enabled.

    :type: bool
    """)

    def _GetHint(self):
        return self._Property('HINT')

    def _SetHint(self, Value):
        self._Property('HINT', tounicode(Value))

    Hint = property(_GetHint, _SetHint,
    doc="""Menu item hint text.

    :type: unicode
    """)

    def _GetId(self):
        return self._Id

    Id = property(_GetId,
    doc="""Unique menu item Id.

    :type: unicode
    """)



"""
Low level *Skype for Linux* interface implemented using *dbus-python* package.

This module handles the options that you can pass to `Skype.__init__`
for Linux machines when the transport is set to *DBus*. See below.

- ``RunMainLoop`` (bool) - If set to False, Skype4Py won't start the GLib main
  loop. Otherwise it is started in a separate thread. The loop must be running for
  Skype4Py events to work properly. Set this option to False if you plan to run the
  loop yourself or if, for example, your GUI framework does it for you.

:requires: Skype for Linux 2.0 (beta) or newer.
"""
__docformat__ = 'restructuredtext en'


import sys
import threading
import time
import warnings
import logging

from Skype4Py.api import Command, SkypeAPIBase, \
                         timeout2float, finalize_opts
from Skype4Py.enums import *
from Skype4Py.errors import SkypeAPIError
from Skype4Py.utils import cndexp


__all__ = ['SkypeAPI']


if getattr(sys, 'skype4py_setup', False):
    # we get here if we're building docs; to let the module import without
    # exceptions, we emulate the dbus module using a class:
    class dbus(object):
        class service(object):
            class Object(object):
                pass
            @staticmethod
            def method(*args, **kwargs):
                return lambda *args, **kwargs: None
else:
    import dbus
    import dbus.glib
    import dbus.service
    from dbus.mainloop.glib import DBusGMainLoop
    import gobject


class SkypeNotify(dbus.service.Object):
    """DBus object which exports a Notify method. This will be called by Skype for all
    notifications with the notification string as a parameter. The Notify method of this
    class calls in turn the callable passed to the constructor.
    """

    def __init__(self, bus, notify):
        dbus.service.Object.__init__(self, bus, '/com/Skype/Client')
        self.notify = notify

    @dbus.service.method(dbus_interface='com.Skype.API.Client')
    def Notify(self, com):
        self.notify(unicode(com))


class SkypeAPI(SkypeAPIBase):
    def __init__(self, opts):
        self.logger = logging.getLogger('Skype4Py.api.posix_dbus.SkypeAPI')
        SkypeAPIBase.__init__(self)
        self.run_main_loop = opts.pop('RunMainLoop', True)
        system_bus = opts.pop('UseSystemBus', False)
        finalize_opts(opts)
        self.skype_in = self.skype_out = self.dbus_name_owner_watch = None

        # initialize glib multithreading support
        gobject.threads_init()
        dbus.glib.threads_init()

        # dbus-python calls object.__init__() with arguments passed to SessionBus(),
        # this throws a warning on newer Python versions; here we suppress it
        warnings.simplefilter('ignore')
        try:
            if system_bus:
                bus = dbus.SystemBus
            else:
                bus = dbus.SessionBus
            self.bus = bus(mainloop=DBusGMainLoop())
        finally:
            warnings.simplefilter('default')
        
        if self.run_main_loop:
            self.mainloop = gobject.MainLoop()

    def run(self):
        self.logger.info('thread started')
        if self.run_main_loop:
            self.mainloop.run()
        self.logger.info('thread finished')

    def close(self):
        if self.run_main_loop:
            self.mainloop.quit()
        self.skype_in = self.skype_out = None
        if self.dbus_name_owner_watch is not None:
            self.bus.remove_signal_receiver(self.dbus_name_owner_watch)
        self.dbus_name_owner_watch = None
        SkypeAPIBase.close(self)

    def set_friendly_name(self, friendly_name):
        SkypeAPIBase.set_friendly_name(self, friendly_name)
        if self.skype_out:
            self.send_command(Command('NAME %s' % friendly_name))

    def start_watcher(self):
        # starts a signal receiver detecting Skype being closed/opened
        self.dbus_name_owner_watch = self.bus.add_signal_receiver(self.dbus_name_owner_changed,
            'NameOwnerChanged',
            'org.freedesktop.DBus',
            'org.freedesktop.DBus',
            '/org/freedesktop/DBus',
            arg0='com.Skype.API')

    def attach(self, timeout, wait=True):
        self.acquire()
        try:
            try:
                if not self.isAlive():
                    self.start_watcher()
                    self.start()
            except AssertionError:
                pass
            try:
                self.wait = True
                t = threading.Timer(timeout2float(timeout), lambda: setattr(self, 'wait', False))
                if wait:
                    t.start()
                while self.wait:
                    if not wait:
                        self.wait = False
                    try:
                        if not self.skype_out:
                            self.skype_out = self.bus.get_object('com.Skype.API', '/com/Skype')
                        if not self.skype_in:
                            self.skype_in = SkypeNotify(self.bus, self.notify)
                    except dbus.DBusException:
                        if not wait:
                            break
                        time.sleep(1.0)
                    else:
                        break
                else:
                    raise SkypeAPIError('Skype attach timeout')
            finally:
                t.cancel()
            command = Command('NAME %s' % self.friendly_name, '', True, timeout)
            if self.skype_out:
                self.release()
                try:
                    self.send_command(command)
                finally:
                    self.acquire()
            if command.Reply != 'OK':
                self.skype_out = None
                self.set_attachment_status(apiAttachRefused)
                return
            self.set_attachment_status(apiAttachSuccess)
        finally:
            self.release()
        command = Command('PROTOCOL %s' % self.protocol, Blocking=True)
        self.send_command(command)
        self.protocol = int(command.Reply.rsplit(None, 1)[-1])

    def is_running(self):
        try:
            self.bus.get_object('com.Skype.API', '/com/Skype')
            return True
        except dbus.DBusException:
            return False

    def startup(self, minimized, nosplash):
        # options are not supported as of Skype 1.4 Beta for Linux
        if not self.is_running():
            import os
            if os.fork() == 0: # we're child
                os.setsid()
                os.execlp('skype', 'skype')

    def shutdown(self):
        import os
        from signal import SIGINT
        fh = os.popen('ps -o %p --no-heading -C skype')
        pid = fh.readline().strip()
        fh.close()
        if pid:
            os.kill(int(pid), SIGINT)
            self.skype_in = self.skype_out = None

    def send_command(self, command):
        if not self.skype_out:
            self.attach(command.Timeout)
        self.push_command(command)
        self.notifier.sending_command(command)
        cmd = u'#%d %s' % (command.Id, command.Command)
        self.logger.debug('sending %s', repr(cmd))
        if command.Blocking:
            if self.run_main_loop:
                command._event = event = threading.Event()
            else:
                command._loop = loop = gobject.MainLoop()
                command._set = False
        else:
            command._timer = timer = threading.Timer(command.timeout2float(), self.pop_command, (command.Id,))
        try:
            result = self.skype_out.Invoke(cmd)
        except dbus.DBusException, err:
            raise SkypeAPIError(str(err))
        if result.startswith(u'#%d ' % command.Id):
            self.notify(result)
        if command.Blocking:
            if self.run_main_loop:
                event.wait(command.timeout2float())
                if not event.isSet():
                    raise SkypeAPIError('Skype command timeout')
            elif not command._set:
                gobject.timeout_add_seconds(int(command.timeout2float()), loop.quit)
                loop.run()
                if not command._set:
                    raise SkypeAPIError('Skype command timeout')
        else:
            timer.start()

    def notify(self, cmd):
        cmd = unicode(cmd)
        self.logger.debug('received %s', repr(cmd))
        if cmd.startswith(u'#'):
            p = cmd.find(u' ')
            command = self.pop_command(int(cmd[1:p]))
            if command is not None:
                command.Reply = cmd[p + 1:]
                if command.Blocking:
                    if self.run_main_loop:
                        command._event.set()
                    else:
                        command._set = True
                        command._loop.quit()
                else:
                    command._timer.cancel()
                self.notifier.reply_received(command)
            else:
                self.notifier.notification_received(cmd[p + 1:])
        else:
            self.notifier.notification_received(cmd)

    def dbus_name_owner_changed(self, owned, old_owner, new_owner):
        self.logger.debug('received dbus name owner changed')
        if new_owner == '':
            self.skype_out = None
        self.set_attachment_status(cndexp((new_owner == ''),
            apiAttachNotAvailable,
            apiAttachAvailable))



apiAttachAvailable = u'Leitud'
apiAttachNotAvailable = u'Kadunud'
apiAttachPendingAuthorization = u'Autoriseerimine'
apiAttachRefused = u'Keeldumine'
apiAttachSuccess = u'\xdchendatud'
apiAttachUnknown = u'M\xe4\xe4ramata'
budDeletedFriend = u'Kustutatud S\xf5ber'
budFriend = u'S\xf5ber'
budNeverBeenFriend = u'Pole Olnud S\xf5ber'
budPendingAuthorization = u'Ootab Autoriseerimist'
budUnknown = u'M\xe4\xe4ramata'
cfrBlockedByRecipient = u'Blokeeritud vastuv\xf5tja poolt'
cfrMiscError = u'M\xe4\xe4ramata viga'
cfrNoCommonCodec = u'pole \xfchist kodekit'
cfrNoProxyFound = u'Ei leitud l\xfc\xfcsi'
cfrNotAuthorizedByRecipient = u'Helistaja pole autoriseeritud'
cfrRecipientNotFriend = u'K\xf5ne vastuv\xf5tja pole s\xf5ber'
cfrRemoteDeviceError = u'Probleem teise poole heliseadmega'
cfrSessionTerminated = u'\xfchendus katkestatud'
cfrSoundIOError = u'Heli viga'
cfrSoundRecordingError = u'Helisalvestuse viga'
cfrUnknown = u'M\xe4\xe4ramata'
cfrUserDoesNotExist = u'Kasutajat v\xf5i numbrit pole olemas'
cfrUserIsOffline = u"Ta ei ole Skype'i sisse logitud"
chsAllCalls = u'Vana Dialoog'
chsDialog = u'Dialoog'
chsIncomingCalls = u'Multiaksept'
chsLegacyDialog = u'Vana Dialoog'
chsMissedCalls = u'Dialoog'
chsMultiNeedAccept = u'Multiaksept'
chsMultiSubscribed = u'Multiteenus'
chsOutgoingCalls = u'Multiteenus'
chsUnknown = u'M\xe4\xe4ramata'
chsUnsubscribed = u'Tellimata'
clsBusy = u'H\xf5ivatud'
clsCancelled = u'Katkestatud'
clsEarlyMedia = u'M\xe4ngib Muusikat'
clsFailed = u'K\xf5ne kahjuks eba\xf5nnestus!'
clsFinished = u'L\xf5petatud'
clsInProgress = u'Aktiivne k\xf5ne'
clsLocalHold = u'Peatatud Lokaalselt'
clsMissed = u'Vastamata k\xf5ne'
clsOnHold = u'Ootel'
clsRefused = u'Keeldutud'
clsRemoteHold = u'Peatatud Eemal'
clsRinging = u'Heliseb'
clsRouting = u'Ruutimine'
clsTransferred = u'M\xe4\xe4ramata'
clsTransferring = u'M\xe4\xe4ramata'
clsUnknown = u'M\xe4\xe4ramata'
clsUnplaced = u'Pole Helistatud'
clsVoicemailBufferingGreeting = u'Tervituse Laadimine'
clsVoicemailCancelled = u'Katkestatud'
clsVoicemailFailed = u'K\xd5nepost eba\xf5nnestus'
clsVoicemailPlayingGreeting = u'Tervituse M\xe4ngimine'
clsVoicemailRecording = u'K\xf5neposti salvestamine'
clsVoicemailSent = u'Saadetud'
clsVoicemailUploading = u'\xdcleslaadimine'
cltIncomingP2P = u'P2P Sisse'
cltIncomingPSTN = u'PSTN Sisse'
cltOutgoingP2P = u'P2P V\xe4lja'
cltOutgoingPSTN = u'PSTN V\xe4lja'
cltUnknown = u'M\xe4\xe4ramata'
cmeAddedMembers = u'Lisas Osalejad'
cmeCreatedChatWith = u'Tegi Jututoa'
cmeEmoted = u'M\xe4\xe4ramata'
cmeLeft = u'Lahkus'
cmeSaid = u'\xdctles'
cmeSawMembers = u'N\xe4gi Osalejaid'
cmeSetTopic = u'Tegi Pealkirja'
cmeUnknown = u'M\xe4\xe4ramata'
cmsRead = u'Loetud'
cmsReceived = u'Vastuv\xf5etud'
cmsSending = u'Saadab...'
cmsSent = u'Saadetud'
cmsUnknown = u'M\xe4\xe4ramata'
conConnecting = u'\xdchendan'
conOffline = u'V\xe4ljas'
conOnline = u'Sees'
conPausing = u'Paus'
conUnknown = u'M\xe4\xe4ramata'
cusAway = u'Eemal'
cusDoNotDisturb = u'H\xf5ivatud'
cusInvisible = u'N\xe4htamatu'
cusLoggedOut = u'V\xe4ljas'
cusNotAvailable = u'Kaua eemal'
cusOffline = u'V\xe4ljas'
cusOnline = u'Sees'
cusSkypeMe = u'Skype Me'
cusUnknown = u'M\xe4\xe4ramata'
cvsBothEnabled = u'Video Saatmine ja Vastuv\xf5tmine'
cvsNone = u'Video Puudub'
cvsReceiveEnabled = u'Video Vastuv\xf5tmine'
cvsSendEnabled = u'Video Saatmine'
cvsUnknown = u''
grpAllFriends = u'K\xf5ik S\xf5brad'
grpAllUsers = u'K\xf5ik Kasutajad'
grpCustomGroup = u'Kasutaja Grupp'
grpOnlineFriends = u'\xdchendatud S\xf5brad'
grpPendingAuthorizationFriends = u'Autoriseerimise Ootel'
grpProposedSharedGroup = u'Pakutud Jagatud Grupp'
grpRecentlyContactedUsers = u'Hiljutised S\xf5brad'
grpSharedGroup = u'Jagatud Grupp'
grpSkypeFriends = u'Skype S\xf5brad'
grpSkypeOutFriends = u'SkypeOut S\xf5brad'
grpUngroupedFriends = u'Grupeerimata'
grpUnknown = u'M\xe4\xe4ramata'
grpUsersAuthorizedByMe = u'Minu Poolt Autoriseeritud'
grpUsersBlockedByMe = u'Minu Poolt Blokeeritud'
grpUsersWaitingMyAuthorization = u'Ootavad Minu Luba'
leaAddDeclined = u'Tagasil\xfckatud'
leaAddedNotAuthorized = u'Pole Autoriseeritud'
leaAdderNotFriend = u'Pole S\xf5ber'
leaUnknown = u'M\xe4\xe4ramata'
leaUnsubscribe = u'Eemaldus'
leaUserIncapable = u'Kasutaja V\xf5imalused Piiratud'
leaUserNotFound = u'Kasutajat Ei Leitud'
olsAway = u'Eemal'
olsDoNotDisturb = u'H\xf5ivatud'
olsNotAvailable = u'Kaua eemal'
olsOffline = u'V\xe4ljas'
olsOnline = u'Sees'
olsSkypeMe = u'Skype Me'
olsSkypeOut = u'SkypeOut'
olsUnknown = u'M\xe4\xe4ramata'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'Naine'
usexMale = u'Mees'
usexUnknown = u'M\xe4\xe4ramata'
vmrConnectError = u'\xdchenduse Viga'
vmrFileReadError = u'Viga Lugemisel'
vmrFileWriteError = u'Viga Kirjutamisel'
vmrMiscError = u'M\xe4\xe4ramata Viga'
vmrNoError = u'Korras'
vmrNoPrivilege = u'Pole K\xf5neposti Privileegi'
vmrNoVoicemail = u'Pole Sellist K\xf5neposti'
vmrPlaybackError = u'Viga Esitamisel'
vmrRecordingError = u'Viga Salvestamisel'
vmrUnknown = u'M\xe4\xe4ramata'
vmsBlank = u'T\xfchi'
vmsBuffering = u'Kogumine'
vmsDeleting = u'Kustutamine'
vmsDownloading = u'Allalaadimine'
vmsFailed = u'Eba\xf5nnestus'
vmsNotDownloaded = u'Pole Laaditud'
vmsPlayed = u'Esitatud'
vmsPlaying = u'Esitamine'
vmsRecorded = u'Salvestatud'
vmsRecording = u'K\xf5neposti salvestamine'
vmsUnknown = u'M\xe4\xe4ramata'
vmsUnplayed = u'M\xe4ngimata'
vmsUploaded = u'\xdcleslaaditud'
vmsUploading = u'\xdcleslaadimine'
vmtCustomGreeting = u'Kasutaja Tervitus'
vmtDefaultGreeting = u'Vaikimisi Tervitus'
vmtIncoming = u'sissetulev k\xf5nepost'
vmtOutgoing = u'V\xe4ljaminev'
vmtUnknown = u'M\xe4\xe4ramata'
vssAvailable = u'Olemas'
vssNotAvailable = u'Puudub'
vssPaused = u'Peatatud'
vssRejected = u'Tagasil\xfckatud'
vssRunning = u'Kestev'
vssStarting = u'Algab'
vssStopping = u'Peatamine'
vssUnknown = u'M\xe4\xe4ramata'



"""APP2APP protocol.
"""
__docformat__ = 'restructuredtext en'


import threading

from utils import *
from user import *


class Application(Cached):
    """Represents an application in APP2APP protocol. Use `skype.Skype.Application` to instantiate.
    """
    _ValidateHandle = staticmethod(tounicode)

    def __repr__(self):
        return Cached.__repr__(self, 'Name')

    def _Alter(self, AlterName, Args=None):
        return self._Owner._Alter('APPLICATION', self.Name, AlterName, Args)

    def _Init(self):
        self._MakeOwner()

    def _Property(self, PropName, Set=None):
        return self._Owner._Property('APPLICATION', self.Name, PropName, Set)

    def _Connect_ApplicationStreams(self, App, Streams):
        if App == self:
            s = [x for x in Streams if x.PartnerHandle == self._Connect_Username]
            if s:
                self._Connect_Stream[0] = s[0]
                self._Connect_Event.set()

    def Connect(self, Username, WaitConnected=False):
        """Connects application to user.

        :Parameters:
          Username : str
            Name of the user to connect to.
          WaitConnected : bool
            If True, causes the method to wait until the connection is established.

        :return: If ``WaitConnected`` is True, returns the stream which can be used to send the
                 data. Otherwise returns None.
        :rtype: `ApplicationStream` or None
        """
        if WaitConnected:
            self._Connect_Event = threading.Event()
            self._Connect_Stream = [None]
            self._Connect_Username = Username
            self._Connect_ApplicationStreams(self, self.Streams)
            self._Owner.RegisterEventHandler('ApplicationStreams', self._Connect_ApplicationStreams)
            self._Alter('CONNECT', Username)
            self._Connect_Event.wait()
            self._Owner.UnregisterEventHandler('ApplicationStreams', self._Connect_ApplicationStreams)
            try:
                return self._Connect_Stream[0]
            finally:
                del self._Connect_Stream, self._Connect_Event, self._Connect_Username
        else:
            self._Alter('CONNECT', Username)

    def Create(self):
        """Creates the APP2APP application in Skype client.
        """
        self._Owner._DoCommand('CREATE APPLICATION %s' % self.Name)

    def Delete(self):
        """Deletes the APP2APP application in Skype client.
        """
        self._Owner._DoCommand('DELETE APPLICATION %s' % self.Name)

    def SendDatagram(self, Text, Streams=None):
        """Sends datagram to application streams.

        :Parameters:
          Text : unicode
            Text to send.
          Streams : sequence of `ApplicationStream`
            Streams to send the datagram to or None if all currently connected streams should be
            used.
        """
        if Streams is None:
            Streams = self.Streams
        for s in Streams:
            s.SendDatagram(Text)

    def _GetConnectableUsers(self):
        return UserCollection(self._Owner, split(self._Property('CONNECTABLE')))

    ConnectableUsers = property(_GetConnectableUsers,
    doc="""All connectible users.

    :type: `UserCollection`
    """)

    def _GetConnectingUsers(self):
        return UserCollection(self._Owner, split(self._Property('CONNECTING')))

    ConnectingUsers = property(_GetConnectingUsers,
    doc="""All users connecting at the moment.

    :type: `UserCollection`
    """)

    def _GetName(self):
        return self._Handle

    Name = property(_GetName,
    doc="""Name of the application.

    :type: unicode
    """)

    def _GetReceivedStreams(self):
        return ApplicationStreamCollection(self, (x.split('=')[0] for x in split(self._Property('RECEIVED'))))

    ReceivedStreams = property(_GetReceivedStreams,
    doc="""All streams that received data and can be read.

    :type: `ApplicationStreamCollection`
    """)

    def _GetSendingStreams(self):
        return ApplicationStreamCollection(self, (x.split('=')[0] for x in split(self._Property('SENDING'))))

    SendingStreams = property(_GetSendingStreams,
    doc="""All streams that send data and at the moment.

    :type: `ApplicationStreamCollection`
    """)

    def _GetStreams(self):
        return ApplicationStreamCollection(self, split(self._Property('STREAMS')))

    Streams = property(_GetStreams,
    doc="""All currently connected application streams.

    :type: `ApplicationStreamCollection`
    """)


class ApplicationStream(Cached):
    """Represents an application stream in APP2APP protocol.
    """
    _ValidateHandle = str

    def __len__(self):
        return self.DataLength

    def __repr__(self):
        return Cached.__repr__(self, 'Handle')

    def Disconnect(self):
        """Disconnects the stream.
        """
        self.Application._Alter('DISCONNECT', self.Handle)

    close = Disconnect

    def Read(self):
        """Reads data from stream.

        :return: Read data or an empty string if none were available.
        :rtype: unicode
        """
        return self.Application._Alter('READ', self.Handle)

    read = Read

    def SendDatagram(self, Text):
        """Sends datagram to stream.

        :Parameters:
          Text : unicode
            Datagram to send.
        """
        self.Application._Alter('DATAGRAM', '%s %s' % (self.Handle, tounicode(Text)))

    def Write(self, Text):
        """Writes data to stream.

        :Parameters:
          Text : unicode
            Data to send.
        """
        self.Application._Alter('WRITE', '%s %s' % (self.Handle, tounicode(Text)))

    write = Write

    def _GetApplication(self):
        return self._Owner

    Application = property(_GetApplication,
    doc="""Application this stream belongs to.

    :type: `Application`
    """)

    def _GetApplicationName(self):
        return self.Application.Name

    ApplicationName = property(_GetApplicationName,
    doc="""Name of the application this stream belongs to. Same as ``ApplicationStream.Application.Name``.

    :type: unicode
    """)

    def _GetDataLength_GetStreamLength(self, Type):
        for s in split(self.Application._Property(Type)):
            h, i = s.split('=')
            if h == self.Handle:
                return int(i)

    def _GetDataLength(self):
        i = self._GetDataLength_GetStreamLength('SENDING')
        if i is not None:
            return i
        i = self._GetDataLength_GetStreamLength('RECEIVED')
        if i is not None:
            return i
        return 0

    DataLength = property(_GetDataLength,
    doc="""Number of bytes awaiting in the read buffer.

    :type: int
    """)

    def _GetHandle(self):
        return self._Handle

    Handle = property(_GetHandle,
    doc="""Stream handle in u'<Skypename>:<n>' format.

    :type: str
    """)

    def _GetPartnerHandle(self):
        return self.Handle.split(':')[0]

    PartnerHandle = property(_GetPartnerHandle,
    doc="""Skypename of the user this stream is connected to.

    :type: str
    """)


class ApplicationStreamCollection(CachedCollection):
    _CachedType = ApplicationStream



"""
Low level *Skype for Linux* interface.

This module handles the options that you can pass to `Skype.__init__` for Linux machines.
The options include:

- ``Transport`` (str) - Name of a channel used to communicate with the Skype client.
  Currently supported values:
  
  - ``'dbus'`` (default)

    Uses *DBus* thrugh *dbus-python* package.
    This is the default if no transport is specified.

    Look into `Skype4Py.api.posix_dbus` for additional options.

  - ``'x11'``

    Uses *X11* messaging through *Xlib*.

    Look into `Skype4Py.api.posix_x11` module for additional options.
"""
__docformat__ = 'restructuredtext en'


from Skype4Py.errors import SkypeAPIError


__all__ = ['SkypeAPI']


def SkypeAPI(opts):
    trans = opts.pop('Transport', 'dbus')
    if trans == 'dbus':
        from posix_dbus import SkypeAPI
    elif trans == 'x11':
        from posix_x11 import SkypeAPI
    else:
        raise SkypeAPIError('Unknown transport: %s' % trans)
    return SkypeAPI(opts)



"""Conversion between constants and text.
"""
__docformat__ = 'restructuredtext en'


import os

import enums


# Following code is needed when building executable files using py2exe.
# Together with the lang.__init__ it makes sure that all languages
# are included in the package built by py2exe. The tool looks just at
# the imports, it ignores the 'if' statement.
#
# More about py2exe: http://www.py2exe.org/

if False:
    import lang
    

class Conversion(object):
    """Allows conversion between constants and text. Access using `skype.Skype.Convert`.
    """

    def __init__(self, Skype):
        """__init__.

        :Parameters:
          Skype : `Skype`
            Skype object.
        """
        self._Language = ''
        self._Module = None
        self._SetLanguage('en')

    def _TextTo(self, Prefix, Value):
        enum = [z for z in [(y, getattr(enums, y)) for y in [x for x in dir(enums) if x.startswith(Prefix)]] if z[1] == Value]
        if enum:
            return str(Value)
        raise ValueError('Bad text')

    def _ToText(self, Prefix, Value):
        enum = [z for z in [(y, getattr(enums, y)) for y in [x for x in dir(enums) if x.startswith(Prefix)]] if z[1] == Value]
        if enum:
            try:
                return unicode(getattr(self._Module, enum[0][0]))
            except AttributeError:
                pass
        raise ValueError('Bad identifier')

    def AttachmentStatusToText(self, Status):
        """Returns attachment status as text.

        :Parameters:
          Status : `enums`.apiAttach*
            Attachment status.

        :return: Text describing the attachment status.
        :rtype: unicode
        """
        return self._ToText('api', Status)

    def BuddyStatusToText(self, Status):
        """Returns buddy status as text.

        :Parameters:
          Status : `enums`.bud*
            Buddy status.

        :return: Text describing the buddy status.
        :rtype: unicode
        """
        return self._ToText('bud', Status)

    def CallFailureReasonToText(self, Reason):
        """Returns failure reason as text.

        :Parameters:
          Reason : `enums`.cfr*
            Call failure reason.

        :return: Text describing the call failure reason.
        :rtype: unicode
        """
        return self._ToText('cfr', Reason)

    def CallStatusToText(self, Status):
        """Returns call status as text.

        :Parameters:
          Status : `enums`.cls*
            Call status.

        :return: Text describing the call status.
        :rtype: unicode
        """
        return self._ToText('cls', Status)

    def CallTypeToText(self, Type):
        """Returns call type as text.

        :Parameters:
          Type : `enums`.clt*
            Call type.

        :return: Text describing the call type.
        :rtype: unicode
        """
        return self._ToText('clt', Type)

    def CallVideoSendStatusToText(self, Status):
        """Returns call video send status as text.

        :Parameters:
          Status : `enums`.vss*
            Call video send status.

        :return: Text describing the call video send status.
        :rtype: unicode
        """
        return self._ToText('vss', Status)

    def CallVideoStatusToText(self, Status):
        """Returns call video status as text.

        :Parameters:
          Status : `enums`.cvs*
            Call video status.

        :return: Text describing the call video status.
        :rtype: unicode
        """
        return self._ToText('cvs', Status)

    def ChatLeaveReasonToText(self, Reason):
        """Returns leave reason as text.

        :Parameters:
          Reason : `enums`.lea*
            Chat leave reason.

        :return: Text describing the chat leave reason.
        :rtype: unicode
        """
        return self._ToText('lea', Reason)

    def ChatMessageStatusToText(self, Status):
        """Returns message status as text.

        :Parameters:
          Status : `enums`.cms*
            Chat message status.

        :return: Text describing the chat message status.
        :rtype: unicode
        """
        return self._ToText('cms', Status)

    def ChatMessageTypeToText(self, Type):
        """Returns message type as text.

        :Parameters:
          Type : `enums`.cme*
            Chat message type.

        :return: Text describing the chat message type.
        :rtype: unicode
        """
        return self._ToText('cme', Type)

    def ChatStatusToText(self, Status):
        """Returns chatr status as text.

        :Parameters:
          Status : `enums`.chs*
            Chat status.

        :return: Text describing the chat status.
        :rtype: unicode
        """
        return self._ToText('chs', Status)

    def ConnectionStatusToText(self, Status):
        """Returns connection status as text.

        :Parameters:
          Status : `enums`.con*
            Connection status.

        :return: Text describing the connection status.
        :rtype: unicode
        """
        return self._ToText('con', Status)

    def GroupTypeToText(self, Type):
        """Returns group type as text.

        :Parameters:
          Type : `enums`.grp*
            Group type.

        :return: Text describing the group type.
        :rtype: unicode
        """
        return self._ToText('grp', Type)

    def OnlineStatusToText(self, Status):
        """Returns online status as text.

        :Parameters:
          Status : `enums`.ols*
            Online status.

        :return: Text describing the online status.
        :rtype: unicode
        """
        return self._ToText('ols', Status)

    def SmsMessageStatusToText(self, Status):
        """Returns SMS message status as text.

        :Parameters:
          Status : `enums`.smsMessageStatus*
            SMS message status.

        :return: Text describing the SMS message status.
        :rtype: unicode
        """
        return self._ToText('smsMessageStatus', Status)

    def SmsMessageTypeToText(self, Type):
        """Returns SMS message type as text.

        :Parameters:
          Type : `enums`.smsMessageType*
            SMS message type.

        :return: Text describing the SMS message type.
        :rtype: unicode
        """
        return self._ToText('smsMessageType', Type)

    def SmsTargetStatusToText(self, Status):
        """Returns SMS target status as text.

        :Parameters:
          Status : `enums`.smsTargetStatus*
            SMS target status.

        :return: Text describing the SMS target status.
        :rtype: unicode
        """
        return self._ToText('smsTargetStatus', Status)

    def TextToAttachmentStatus(self, Text):
        """Returns attachment status code.

        :Parameters:
          Text : unicode
            Text, one of 'UNKNOWN', 'SUCCESS', 'PENDING_AUTHORIZATION', 'REFUSED', 'NOT_AVAILABLE',
            'AVAILABLE'.

        :return: Attachment status.
        :rtype: `enums`.apiAttach*
        """
        conv = {'UNKNOWN': enums.apiAttachUnknown,
                'SUCCESS': enums.apiAttachSuccess,
                'PENDING_AUTHORIZATION': enums.apiAttachPendingAuthorization,
                'REFUSED': enums.apiAttachRefused,
                'NOT_AVAILABLE': enums.apiAttachNotAvailable,
                'AVAILABLE': enums.apiAttachAvailable}
        try:
            return self._TextTo('api', conv[Text.upper()])
        except KeyError:
            raise ValueError('Bad text')

    def TextToBuddyStatus(self, Text):
        """Returns buddy status code.

        :Parameters:
          Text : unicode
            Text, one of 'UNKNOWN', 'NEVER_BEEN_FRIEND', 'DELETED_FRIEND', 'PENDING_AUTHORIZATION',
            'FRIEND'.

        :return: Buddy status.
        :rtype: `enums`.bud*
        """
        conv = {'UNKNOWN': enums.budUnknown,
                'NEVER_BEEN_FRIEND': enums.budNeverBeenFriend,
                'DELETED_FRIEND': enums.budDeletedFriend,
                'PENDING_AUTHORIZATION': enums.budPendingAuthorization,
                'FRIEND': enums.budFriend}
        try:
            return self._TextTo('bud', conv[Text.upper()])
        except KeyError:
            raise ValueError('Bad text')

    def TextToCallStatus(self, Text):
        """Returns call status code.

        :Parameters:
          Text : unicode
            Text, one of `enums`.cls*.

        :return: Call status.
        :rtype: `enums`.cls*

        :note: Currently, this method only checks if the given string is one of the allowed ones and
               returns it or raises a ``ValueError``.
        """
        return self._TextTo('cls', Text)

    def TextToCallType(self, Text):
        """Returns call type code.

        :Parameters:
          Text : unicode
            Text, one of `enums`.clt*.

        :return: Call type.
        :rtype: `enums`.clt*

        :note: Currently, this method only checks if the given string is one of the allowed ones and
               returns it or raises a ``ValueError``.
        """
        return self._TextTo('clt', Text)

    def TextToChatMessageStatus(self, Text):
        """Returns message status code.

        :Parameters:
          Text : unicode
            Text, one of `enums`.cms*.

        :return: Chat message status.
        :rtype: `enums`.cms*

        :note: Currently, this method only checks if the given string is one of the allowed ones and
               returns it or raises a ``ValueError``.
        """
        return self._TextTo('cms', Text)

    def TextToChatMessageType(self, Text):
        """Returns message type code.

        :Parameters:
          Text : unicode
            Text, one of `enums`.cme*.

        :return: Chat message type.
        :rtype: `enums`.cme*

        :note: Currently, this method only checks if the given string is one of the allowed ones and
               returns it or raises a ``ValueError``.
        """
        return self._TextTo('cme', Text)

    def TextToConnectionStatus(self, Text):
        """Retunes connection status code.

        :Parameters:
          Text : unicode
            Text, one of `enums`.con*.

        :return: Connection status.
        :rtype: `enums`.con*

        :note: Currently, this method only checks if the given string is one of the allowed ones and
               returns it or raises a ``ValueError``.
        """
        return self._TextTo('con', Text)

    def TextToGroupType(self, Text):
        """Returns group type code.

        :Parameters:
          Text : unicode
            Text, one of `enums`.grp*.

        :return: Group type.
        :rtype: `enums`.grp*

        :note: Currently, this method only checks if the given string is one of the allowed ones and
               returns it or raises a ``ValueError``.
        """
        return self._TextTo('grp', Text)

    def TextToOnlineStatus(self, Text):
        """Returns online status code.

        :Parameters:
          Text : unicode
            Text, one of `enums`.ols*.

        :return: Online status.
        :rtype: `enums`.ols*

        :note: Currently, this method only checks if the given string is one of the allowed ones and
               returns it or raises a ``ValueError``.
        """
        return self._TextTo('ols', Text)

    def TextToUserSex(self, Text):
        """Returns user sex code.

        :Parameters:
          Text : unicode
            Text, one of `enums`.usex*.

        :return: User sex.
        :rtype: `enums`.usex*

        :note: Currently, this method only checks if the given string is one of the allowed ones and
               returns it or raises a ``ValueError``.
        """
        return self._TextTo('usex', Text)

    def TextToUserStatus(self, Text):
        """Returns user status code.

        :Parameters:
          Text : unicode
            Text, one of `enums`.cus*.

        :return: User status.
        :rtype: `enums`.cus*

        :note: Currently, this method only checks if the given string is one of the allowed ones and
               returns it or raises a ``ValueError``.
        """
        return self._TextTo('cus', Text)

    def TextToVoicemailStatus(self, Text):
        """Returns voicemail status code.

        :Parameters:
          Text : unicode
            Text, one of `enums`.vms*.

        :return: Voicemail status.
        :rtype: `enums`.vms*

        :note: Currently, this method only checks if the given string is one of the allowed ones and
               returns it or raises a ``ValueError``.
        """
        return self._TextTo('vms', Text)

    def UserSexToText(self, Sex):
        """Returns user sex as text.

        :Parameters:
          Sex : `enums`.usex*
            User sex.

        :return: Text describing the user sex.
        :rtype: unicode
        """
        return self._ToText('usex', Sex)

    def UserStatusToText(self, Status):
        """Returns user status as text.

        :Parameters:
          Status : `enums`.cus*
            User status.

        :return: Text describing the user status.
        :rtype: unicode
        """
        return self._ToText('cus', Status)

    def VoicemailFailureReasonToText(self, Reason):
        """Returns voicemail failure reason as text.

        :Parameters:
          Reason : `enums`.vmr*
            Voicemail failure reason.

        :return: Text describing the voicemail failure reason.
        :rtype: unicode
        """
        return self._ToText('vmr', Reason)

    def VoicemailStatusToText(self, Status):
        """Returns voicemail status as text.

        :Parameters:
          Status : `enums`.vms*
            Voicemail status.

        :return: Text describing the voicemail status.
        :rtype: unicode
        """
        return self._ToText('vms', Status)

    def VoicemailTypeToText(self, Type):
        """Returns voicemail type as text.

        :Parameters:
          Type : `enums`.vmt*
            Voicemail type.

        :return: Text describing the voicemail type.
        :rtype: unicode
        """
        return self._ToText('vmt', Type)

    def _GetLanguage(self):
        return self._Language

    def _SetLanguage(self, Language):
        try:
            self._Module = __import__('lang.%s' % Language, globals(), locals(), ['lang'])
            self._Language = str(Language)
        except ImportError:
            raise ValueError('Unknown language: %s' % Language)

    Language = property(_GetLanguage, _SetLanguage,
    doc="""Language used for all "ToText" conversions.

    Currently supported languages: ar, bg, cs, cz, da, de, el, en, es, et, fi, fr, he, hu, it, ja, ko,
    lt, lv, nl, no, pl, pp, pt, ro, ru, sv, tr, x1.

    :type: str
    """)
